import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.maintenance_time_type import MaintenanceTimeType
from ..types import UNSET, Unset

T = TypeVar("T", bound="MaintenanceTime")


@_attrs_define
class MaintenanceTime:
    """
    Attributes:
        type (MaintenanceTimeType): Type of the maintenance time
        start_date (Union[Unset, datetime.datetime]): Start time of the time configuration of maintenance
        end_date (Union[Unset, datetime.datetime]): End time of the time configuration of maintenance
    """

    type: MaintenanceTimeType
    start_date: Union[Unset, datetime.datetime] = UNSET
    end_date: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type.value

        start_date: Union[Unset, str] = UNSET
        if not isinstance(self.start_date, Unset):
            start_date = self.start_date.isoformat()

        end_date: Union[Unset, str] = UNSET
        if not isinstance(self.end_date, Unset):
            end_date = self.end_date.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
            }
        )
        if start_date is not UNSET:
            field_dict["startDate"] = start_date
        if end_date is not UNSET:
            field_dict["endDate"] = end_date

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        type = MaintenanceTimeType(d.pop("type"))

        _start_date = d.pop("startDate", UNSET)
        start_date: Union[Unset, datetime.datetime]
        if isinstance(_start_date, Unset):
            start_date = UNSET
        else:
            start_date = isoparse(_start_date)

        _end_date = d.pop("endDate", UNSET)
        end_date: Union[Unset, datetime.datetime]
        if isinstance(_end_date, Unset):
            end_date = UNSET
        else:
            end_date = isoparse(_end_date)

        maintenance_time = cls(
            type=type,
            start_date=start_date,
            end_date=end_date,
        )

        maintenance_time.additional_properties = d
        return maintenance_time

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ServiceIncidentStakeholderProperties")


@_attrs_define
class ServiceIncidentStakeholderProperties:
    """
    Attributes:
        message (str): Message that is to be passed to audience that is generally used to provide a content information
            about the alert.
        enable (Union[Unset, bool]): Option to enable stakeholder notifications.Default value is true.
        description (Union[Unset, str]): Description that is generally used to provide a detailed information about the
            alert.
    """

    message: str
    enable: Union[Unset, bool] = UNSET
    description: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        message = self.message

        enable = self.enable

        description = self.description

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "message": message,
            }
        )
        if enable is not UNSET:
            field_dict["enable"] = enable
        if description is not UNSET:
            field_dict["description"] = description

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        message = d.pop("message")

        enable = d.pop("enable", UNSET)

        description = d.pop("description", UNSET)

        service_incident_stakeholder_properties = cls(
            message=message,
            enable=enable,
            description=description,
        )

        service_incident_stakeholder_properties.additional_properties = d
        return service_incident_stakeholder_properties

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.team_log_details import TeamLogDetails


T = TypeVar("T", bound="TeamLog")


@_attrs_define
class TeamLog:
    """
    Attributes:
        offset (Union[Unset, str]):
        logs (Union[Unset, List['TeamLogDetails']]):
    """

    offset: Union[Unset, str] = UNSET
    logs: Union[Unset, List["TeamLogDetails"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        offset = self.offset

        logs: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.logs, Unset):
            logs = []
            for logs_item_data in self.logs:
                logs_item = logs_item_data.to_dict()
                logs.append(logs_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if offset is not UNSET:
            field_dict["offset"] = offset
        if logs is not UNSET:
            field_dict["logs"] = logs

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.team_log_details import TeamLogDetails

        d = src_dict.copy()
        offset = d.pop("offset", UNSET)

        logs = []
        _logs = d.pop("logs", UNSET)
        for logs_item_data in _logs or []:
            logs_item = TeamLogDetails.from_dict(logs_item_data)

            logs.append(logs_item)

        team_log = cls(
            offset=offset,
            logs=logs,
        )

        team_log.additional_properties = d
        return team_log

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ListAlertsSort(str, Enum):
    ACKNOWLEDGED = "acknowledged"
    ALIAS = "alias"
    COUNT = "count"
    CREATEDAT = "createdAt"
    INTEGRATION_NAME = "integration.name"
    INTEGRATION_TYPE = "integration.type"
    ISSEEN = "isSeen"
    LASTOCCURREDAT = "lastOccurredAt"
    MESSAGE = "message"
    OWNER = "owner"
    REPORT_ACKNOWLEDGEDBY = "report.acknowledgedBy"
    REPORT_ACKTIME = "report.ackTime"
    REPORT_CLOSEDBY = "report.closedBy"
    REPORT_CLOSETIME = "report.closeTime"
    SNOOZED = "snoozed"
    SNOOZEDUNTIL = "snoozedUntil"
    SOURCE = "source"
    STATUS = "status"
    TINYID = "tinyId"
    UPDATEDAT = "updatedAt"

    def __str__(self) -> str:
        return str(self.value)



import asyncio
import re
import time
import webbrowser
from math import ceil
from typing import TYPE_CHECKING

from desktop_notifier import Urgency
from textual import on, work
from textual.app import ComposeResult
from textual.binding import Binding
from textual.containers import Container, Vertical
from textual.content import Content
from textual.css.query import NoMatches
from textual.message import Message
from textual.reactive import reactive
from textual.widgets import (
    ContentSwitcher,
    DataTable,
    Footer,
    Input,
    Label,
    Markdown,
    Pretty,
)
from textual.widgets._data_table import CellDoesNotExist
from udatetime import from_string as dtfstr

import tygenie.config as config
import tygenie.opsgenie as opsgenie
from tygenie import consts
from tygenie.screen import TyScreen
from tygenie.widgets.alert_actions import AlertActionContainer
from tygenie.widgets.alert_details import (
    AlertDetails,
    AlertDetailsTabbedContent,
    AlertDetailTitle,
)
from tygenie.widgets.alert_notes import AlertNotes
from tygenie.widgets.center_middle import CenterMiddle
from tygenie.widgets.datatable import TygenieDataTable
from tygenie.widgets.tags import Tags

if TYPE_CHECKING:
    from tygenie.app import TygenieApp


class AlertsScreen(TyScreen):

    DEFAULT_CSS = """\
    AlertsScreen{
        & #no_alerts_list_label_container {
            height: 1fr;
            hatch: right $surface-lighten-1 70%;
        }
    }
"""

    kb = {
        "ack": "a",
        "unack": "u",
        "close": "c",
        "tag": "t",
        "untag": "T",
        "refresh": "r",
        "next_page": "right",
        "previous_page": "left",
        "add_note": "n",
        "open_in_webbrowser": "ctrl+w",
        "focus_on_alerts_list": "f5",
        "focus_on_alert_details": "f6",
        "focus_on_alert_notes": "f7",
    }

    BINDINGS = [
        Binding(key=kb["ack"], id="ack", action="post_ack_alert()", description="Ack"),
        Binding(
            key=kb["unack"],
            id="unack",
            action="post_unack_alert()",
            description="Unack",
        ),
        Binding(
            key=kb["close"],
            id="close",
            action="post_close_alert()",
            description="Close",
        ),
        Binding(key=kb["tag"], id="tag", action="post_tag_alert()", description="Tag"),
        Binding(
            key=kb["untag"],
            id="untag",
            action="post_remove_tag_alert()",
            description="Untag",
        ),
        Binding(
            key=kb["refresh"],
            id="refresh",
            action="manual_refresh()",
            description="Refresh",
        ),
        Binding(
            key=kb["add_note"],
            id="add_note",
            action="open_add_note()",
            description="Add note",
        ),
        Binding(
            key=kb["previous_page"],
            id="previous_page",
            action="previous_page()",
            description="Prev page",
            show=True,
        ),
        Binding(
            key=kb["next_page"],
            id="next_page",
            action="next_page()",
            description="Next page",
            show=True,
        ),
        Binding(
            key=kb["open_in_webbrowser"],
            id="open_in_webbrowser",
            action="open_in_webbrowser()",
            description="In Web Browser",
        ),
        Binding(
            key=kb["focus_on_alerts_list"],
            id="focus_on_alerts_list",
            action="alerts_list_focus()",
            show=False,
        ),
        Binding(
            key=kb["focus_on_alert_details"],
            id="focus_on_alert_details",
            action="alert_details_focus()",
            show=False,
        ),
        Binding(
            key=kb["focus_on_alert_notes"],
            id="focus_on_alert_notes",
            action="alert_notes_focus()",
            show=False,
        ),
        Binding(
            key="ctrl+down",
            id="increase_height",
            action="increase_height()",
            show=False,
        ),
        Binding(
            key="ctrl+up",
            id="decrease_height",
            action="decrease_height()",
            show=False,
        ),
    ]

    def action_increase_height(self):
        content = self.query_one("#alerts_list_switcher", ContentSwitcher)
        match = re.match(r"^(\d+)(\%|vw|vh|w|h)$", str(content.styles.height))
        if match is not None:
            numeric_height = int(match.group(1))
            unit_height = match.group(2)
            content.styles.height = f"{int(numeric_height)+2}{unit_height}"
        else:
            self.notify(
                severity="error",
                message=f'Wrong height style format: "{content.styles.height}"',
            )

    def action_decrease_height(self):
        content = self.query_one("#alerts_list_switcher", ContentSwitcher)
        match = re.match(r"^(\d+)(\%|vw|vh|w|h)$", str(content.styles.height))
        if match is not None:
            numeric_height = int(match.group(1))
            unit_height = match.group(2)
            content.styles.height = f"{int(numeric_height)-2}{unit_height}"
        else:
            self.notify(
                severity="error",
                message=f'Wrong height style format: "{content.styles.height}"',
            )

    class AckAlert(Message):
        """A message to ack the selected alert"""

    class UnackAlert(Message):
        """A message to un-ack the selected alert"""

    class CloseAlert(Message):
        """A message to close the selected alert"""

    class GetAlertsList(Message):
        """A message to fetch alerts list"""

        def __init__(
            self,
            next: bool = False,
            previous: bool = False,
            filter_name: str | None = None,
        ):
            self.previous = previous
            self.next = next
            self.filter_name = filter_name
            super().__init__()

    class GetWhoisOnCall(Message):
        """A message to get the team member who is on call"""

    class GetAlertsCount(Message):
        """A message to get total count of alerts matching the current filter"""

        def __init__(self, parameters: dict = {}):
            self.parameters = parameters
            super().__init__()

    class TagAlert(Message):
        """A message to tag the selected alert"""

    class RemoveTagAlert(Message):
        """A message to remove tag of the selected alert"""

    class AddNote(Message):
        """A message to add note on selected alert"""

        def __init__(self, opsgenie_id: str = "", note: str = ""):
            self.opsgenie_id = opsgenie_id
            self.note = note
            super().__init__()

    class DesktopNotify(Message):
        """A message to send destop notification"""

        def __init__(self, result={}):
            self.result = result
            super().__init__()

    class LookupDataWithDelay(Message):
        """A message to proceed to getting alert list after a delay"""

        def __init__(self, delay: float = 1.75) -> None:
            self.delay = delay
            super().__init__()

    class NotifyMessage(Message):
        """A message to send in-app notification"""

        def __init__(self, message="", severity="information"):
            self.severity = severity
            self.message = message
            super().__init__()

    data = {}
    opsgenie_query = opsgenie.Query()
    page = reactive(1)
    first_alert_count = reactive(0)
    last_alert_count = reactive(0)
    current_filter = reactive("")
    total_alerts = reactive(0)
    lookup_count_rows = reactive(0)
    current_on_call_member = reactive("")

    def __init__(self):
        super().__init__()
        self.app: "TygenieApp"

    async def reload(self):
        await self.recompose()

    @work(exclusive=True, exit_on_error=True, thread=True)
    async def on_screen_resume(self):
        await self.lookup_data()

    def _update_paging_label(self):
        label = (
            f"[$accent]Page:[/] [b]{self.page}/{ceil(self.total_alerts / self.opsgenie_query.limit)}[/b]"
            f" | [$accent]Displayed:[/] [b]{self.first_alert_count}-{self.last_alert_count}/{self.total_alerts}[/b]"
            f' | [$accent]Filter:[/] [b]{config.ty_config.tygenie.get("filters", "")[self.opsgenie_query.current_filter]["description"]}[/b]'
        )

        if config.ty_config.opsgenie.get("on_call_schedule_ids"):
            label += f" | [$accent]OnCall:[/] [b]{self.current_on_call_member}[/b]"

        self.query_one("#alerts_list_switcher", ContentSwitcher).border_subtitle = (
            Content.from_markup(label)
        )

    def update_paging_label(self):
        try:
            self.query_one("#alerts_list_switcher")
        except NoMatches:
            pass
        else:
            self._update_paging_label()

    def _update_no_alert_label(self):
        filter = config.ty_config.tygenie.get("filters", "")[
            self.opsgenie_query.current_filter
        ]
        content = f"There is no alert for filter '[b]{filter['description']}[/b]' ([i]{filter['filter']}[/i])"
        self.query_one("#no_alerts_list_label", Label).update(
            content=Content.from_markup(content)
        )

    def update_no_alert_label(self):
        try:
            self.query_one("#no_alerts_list_label", Label)
        except NoMatches:
            pass
        else:
            self._update_no_alert_label()

    def watch_page(self):
        self.update_paging_label()

    def watch_current_filter(self):
        self.update_paging_label()

    def watch_total_alerts(self):
        self.update_paging_label()
        self.update_no_alert_label()

    def watch_lookup_count_rows(self):
        self.update_paging_label()

    def watch_opsgenie_query_offset(self):
        self.update_paging_label()

    def watch_first_alert_count(self):
        self.update_paging_label()

    def watch_last_alert_count(self):
        self.update_paging_label()

    def watch_current_on_call_member(self):
        self.update_paging_label()

    def _set_custom_bindings(self):
        self.app.set_keymap(
            {
                k: v
                for k, v in config.ty_config.tygenie.get("keybindings", {}).items()
                if k != "palette"
            }
        )

    def _add_filters_to_bindings(self):
        filters = config.ty_config.tygenie.get("filters", {})
        for filter_name, value in filters.items():
            self.bind(
                value["key"],
                action=f"lookup_data('{filter_name}')",
                description=f"Filter: {value['description']}",
                key_display=value["key"],
            )
        self.refresh_bindings()

    def compose(self) -> ComposeResult:
        content_switcher = ContentSwitcher(initial=None, id="alerts_list_switcher")
        content_switcher.border_title = "Alerts list"
        content_switcher.focus()

        with content_switcher:
            yield CenterMiddle(
                Label(
                    "",
                    id="no_alerts_list_label",
                ),
                id="no_alerts_list_label_container",
            )
            with Vertical(id="alerts_container"):
                yield TygenieDataTable(id="alerts_data_table")

        yield Container(
            AlertDetails(id="raw_alert_details"),
            AlertNotes(id="alert_notes"),
            AlertActionContainer(id="alert_action_container"),
            id="alert_detail_container",
        )
        self._set_custom_bindings()
        self._add_filters_to_bindings()

        footer = Footer(id="footer")
        footer.compact = True
        yield footer

    def on_data_table_row_highlighted(self, event):
        event.stop(stop=True)
        if not re.match(r"data_table_", event.row_key.value):
            return
        # Check if in config we enabled open_detail_alert_on_enter
        open_detail_alert_on_enter = config.ty_config.tygenie.get(
            "open_detail_alert_on_enter", False
        )
        if not open_detail_alert_on_enter:
            self.open_detail_and_note_alert()

    def on_data_table_row_selected(self, event):
        event.stop(stop=True)
        # Check if in config we enabled open_detail_alert_on_enter
        open_detail_alert_on_enter = config.ty_config.tygenie.get(
            "open_detail_alert_on_enter", False
        )
        if not open_detail_alert_on_enter:
            self.open_detail_and_note_alert()

    @work(exclusive=True, exit_on_error=True, thread=False)
    async def open_detail_and_note_alert(self):
        alert_note = self.query_one("#alert_notes", AlertNotes)
        raw_alert_details = self.query_one("#raw_alert_details", AlertDetails)
        opsgenie_id = self._get_cursor_opsgenie_id()
        if opsgenie_id is None:
            self._notify_invalid_opsgenie_id()
            return
        asyncio.create_task(alert_note.get_alert_notes(opsgenie_id))
        asyncio.create_task(raw_alert_details.get_alert_detail(opsgenie_id))

    @on(GetWhoisOnCall)
    async def get_whois_on_call(self, message: GetWhoisOnCall):
        on_call_schedule_ids = config.ty_config.opsgenie.get("on_call_schedule_ids", [])
        self.current_on_call_member = ""
        for on_call_schedule_id in on_call_schedule_ids:
            result = await opsgenie.client.api.whois_on_call(
                parameters={"identifier": on_call_schedule_id}
            )
            if result is not None and len(result.data.on_call_recipients):
                self.current_on_call_member = re.sub(
                    "@.*$", "", result.data.on_call_recipients[0]
                )
                break
            else:
                if on_call_schedule_id == on_call_schedule_ids[-1]:
                    self.post_message(
                        self.NotifyMessage(
                            severity="error",
                            message=f"Unable to find oncall user: {result}",
                        )
                    )

    @on(GetAlertsList)
    async def _get_alerts_list_from_message(self, message: GetAlertsList):
        await self._get_alerts_list(
            previous=message.previous,
            next=message.next,
            filter_name=message.filter_name,
        )

    async def _get_alerts_list(
        self,
        previous: bool = False,
        next: bool = False,
        filter_name: str | None = None,
    ):
        parameters = self.opsgenie_query.current
        new_filter = False

        if filter_name is None:
            filter_name = self.opsgenie_query.current_filter
        elif filter_name != self.opsgenie_query.current_filter:
            new_filter = True
            # reset offset in case we are changing filter
            self.opsgenie_query.offset = 0

        if filter_name is not None:
            parameters = self.opsgenie_query.get(filter_name=filter_name)

        if previous is True:
            parameters = self.opsgenie_query.get_previous()
        elif next is True:
            parameters = self.opsgenie_query.get_next()

        self.opsgenie_query.current = parameters

        def _update_data_table(alerts=None):
            alerts_list_switcher = self.query_one(
                "#alerts_list_switcher", ContentSwitcher
            )

            if alerts is None:
                alerts_list_switcher.current = "no_alerts_list_label_container"
                return

            if not alerts.data and (previous or next):
                # keep displayed data because previous or next returned no more data
                return

            if not alerts.data and not (previous or next):
                # there is no alert, switch content
                alerts_list_switcher.current = "no_alerts_list_label_container"
                return

            table = self.query_one("#alerts_data_table", TygenieDataTable)
            table.cursor_type = "row"
            table.zebra_stripes = True
            table.show_header = True
            table.show_cursor = True
            table.cell_padding = 1

            # it means we change filter
            if filter_name is not None:
                table.focus(True)

            if not table.columns:
                table.add_columns(
                    *self.app.alerts_list_formatter.displayed_fields.values()
                )
                cursor_opsgenie_id = 0
            else:  # table exist, let's keep current selected alert
                cursor_opsgenie_id = self._get_cursor_opsgenie_id() or 0

            # Store current rows indexes to build a notifiication
            previous_rows = [r.key.value for r in table.rows.values()]
            rows_messages = []
            current_row = table.cursor_row
            row_coordinate = 0

            table.clear()

            self.page = self.opsgenie_query.current_page()
            self.current_filter = self.opsgenie_query.current_filter

            # display alerts with highest priority first
            # only sort the current page
            if config.ty_config.tygenie.get("alerts", {}).get(
                "display_page_by_priority", False
            ):
                alerts = sorted(alerts.data, key=lambda alert: alert.priority)
            else:
                alerts = alerts.data

            for index, alert in enumerate(alerts):
                key = f"data_table_{alert.id}"
                self.app.alerts_list_formatter.alert = alert
                formatted_values = self.app.alerts_list_formatter.format()
                table.add_row(
                    *tuple(
                        [
                            formatted_values[f]
                            for f in self.app.alerts_list_formatter.displayed_fields.keys()
                        ]
                    ),
                    key=key,
                )
                # If this is the alert selected before the update, keep coordinates
                # to be the one selected
                if alert.id == cursor_opsgenie_id:
                    row_coordinate = index

                # Check if this is a new alert, in such case keep the message only
                # It works if we are on first page of data
                if self.page == 1 and key not in previous_rows:
                    rows_messages.append(alert.message)

            # Compute the highlighted row depending the mode we are
            # If previous or next or new_filter => 0
            if previous:
                row_coordinate = len(alerts) - 1
            if next or new_filter:
                row_coordinate = 0
            # We did not find the alert previously highlighted
            elif row_coordinate == 0:
                if current_row <= len(alerts):
                    row_coordinate = current_row
                else:
                    row_coordinate = len(alerts) - 1

            # Move cursor to the correct alert
            table.move_cursor(row=row_coordinate)

            alerts_list_switcher.current = "alerts_container"

            result = {"rows_messages": rows_messages}
            if self.app.started and not new_filter and not next and not previous:
                self.post_message(self.DesktopNotify(result=result))

            if not self.app.started:
                # First time data are lookup, now we can flag the app as started
                self.app.started = True

        alerts = await opsgenie.client.api.list_alerts(parameters=parameters)

        if alerts is not None:
            _update_data_table(alerts)
            self._update_paging_label()
            self.lookup_count_rows = len(alerts.data)
            if not len(alerts.data) and (next or previous):
                pass
            else:
                self.first_alert_count = self.opsgenie_query.offset + 1
                self.last_alert_count = (
                    self.opsgenie_query.offset + self.lookup_count_rows
                )
        else:
            self.post_message(
                self.NotifyMessage(message="Unable to get alert list", severity="error")
            )

    @on(GetAlertsCount)
    async def get_alerts_count(self, message):
        parameters = self.opsgenie_query.current
        result = await opsgenie.client.api.count_alerts(parameters=parameters)
        if result is not None:
            self.total_alerts = result.data.count
        else:
            self.post_message(
                self.NotifyMessage(
                    severity="error", message="Unable to get alerts count"
                )
            )

    async def on_mount(self) -> None:
        self.send_lookup_data()
        refresh_period = max(60, config.ty_config.tygenie.get("refresh_period", 300))
        self.set_interval(refresh_period, self.send_lookup_data)

    def send_lookup_data(self):
        self.post_message(self.LookupDataWithDelay(delay=0))

    async def _lookup_data(self, filter_name=None, previous=False, next=False):
        # Do not lookup in case we have the modal AddNote screen pop up to
        # prevent adding a note to the wrong alert
        # (in case the alert disappear)
        if self.app.screen.name == consts.ADD_NOTE_SCREEN_NAME:
            return

        self.post_message(
            self.GetAlertsList(
                next=next,
                previous=previous,
                filter_name=filter_name,
            )
        )
        self.post_message(self.GetAlertsCount())
        self.post_message(self.GetWhoisOnCall())

    @on(DesktopNotify)
    async def desktop_notify(self, event):
        result = event.result
        dn_config = config.ty_config.tygenie.get("desktop_notification", {})
        if dn_config.get("enable", True) is True and len(result["rows_messages"]):
            if (
                dn_config.get("when_on_call_only", False) is True
                and re.sub("@.+$", "", config.ty_config.opsgenie.get("username", ""))
                != self.current_on_call_member
            ):
                # Nothing to do unless we are on call
                return

            # Urgency can be: Critical, Normal, Low
            urgency = str(dn_config.get("urgency", "Critical")).capitalize()
            messages = [f"- {r}" for r in result["rows_messages"]]
            notifier = self.app.notifier
            await notifier.send(
                title=f'Tygenie: {len(result["rows_messages"])} new alerts',
                message="\n".join(messages),
                urgency=getattr(Urgency, urgency),
            )

    async def lookup_data(self, filter_name=None):
        await self._lookup_data(filter_name=filter_name)

    @on(LookupDataWithDelay)
    @work(exclusive=True, exit_on_error=True, thread=True)
    async def lookup_data_with_delay(self, event: LookupDataWithDelay | None = None):
        if event is not None:
            time.sleep(event.delay)
        await self._lookup_data()

    # Method useed for manual refresh
    async def action_manual_refresh(self):
        self.notify("Refreshing data...")
        await self.lookup_data()

    async def action_lookup_data(self, filter_name=None):
        # Refreshing because want to filter on an other filter
        if filter_name is not None:
            self.notify(
                f"Refreshing data with filter '{filter_name}'",
                title="Filtering alerts",
                severity="information",
            )
        await self.lookup_data(filter_name=filter_name)

    @on(NotifyMessage)
    def notify_message(self, message):
        self.notify(severity=message.severity, message=message.message)

    @on(TygenieDataTable.NextPage)
    @work(exclusive=True, exit_on_error=False, thread=False)
    async def action_next_page(self, event: TygenieDataTable.NextPage | None = None):
        await self._get_alerts_list(next=True)
        if not self.lookup_count_rows:
            # reset get_next by using get_previous
            self.opsgenie_query.get_previous()
            self.post_message(self.NotifyMessage(message="No more data to fetch"))
        else:
            self.page = self.opsgenie_query.current_page()
            self.post_message(self.NotifyMessage(message=f"Page {self.page} loaded"))

    @on(TygenieDataTable.PreviousPage)
    @work(exclusive=True, exit_on_error=False, thread=False)
    async def action_previous_page(
        self, event: TygenieDataTable.PreviousPage | None = None
    ):
        if self.opsgenie_query.current_page() == 1:
            self.post_message(
                self.NotifyMessage(message="You are already on first page")
            )
        else:
            await self._get_alerts_list(previous=True)
            if not self.lookup_count_rows:
                # reset get_previous by using get_next
                self.opsgenie_query.get_next()
            else:
                self.page = self.opsgenie_query.current_page()
                self.post_message(
                    self.NotifyMessage(message=f"Page {self.page} loaded")
                )

    def _get_note_on_action(self, action):
        note_on_action = config.ty_config.tygenie.get(f"note_on_{action}", {})
        if note_on_action["enable"] is False:
            return ""
        return note_on_action["message"]

    def action_post_ack_alert(self):
        self.post_message(self.AckAlert())

    @on(AckAlert)
    @work(exclusive=True, exit_on_error=False, thread=False)
    async def ack_alert(self):
        # Check if we can ack
        current_status = self._get_cursor_status()
        if str(current_status) != "open":
            self.notify(
                severity="warning",
                message=f'Alert can not be acked because in status "{current_status}"',
            )
            return

        opsgenie_id = self._get_cursor_opsgenie_id()
        if opsgenie_id is None:
            self._notify_invalid_opsgenie_id()
            return

        message = self._get_cursor_message()
        self.notify(f"Acking alert\n{message}")
        await opsgenie.client.api.ack_alert(
            parameters={"identifier": opsgenie_id},
            note=self._get_note_on_action("ack"),
        )
        self.post_message(self.LookupDataWithDelay(delay=1.75))

    def action_post_unack_alert(self):
        self.post_message(self.UnackAlert())

    @on(UnackAlert)
    @work(exclusive=True, exit_on_error=False, thread=False)
    async def unack_alert(self):
        current_status = self._get_cursor_status()
        if str(current_status) != "acked":
            self.notify(
                severity="warning",
                message=f'Alert can not be unacked because in status "{current_status}"',
            )
            return

        opsgenie_id = self._get_cursor_opsgenie_id()
        if opsgenie_id is None:
            self._notify_invalid_opsgenie_id()
            return

        message = self._get_cursor_message()
        self.notify(f"Unacking alert\n{message} ...")
        await opsgenie.client.api.unack_alert(
            parameters={"identifier": opsgenie_id},
            note=self._get_note_on_action("unack"),
        )
        self.post_message(self.LookupDataWithDelay(delay=1.75))

    def action_post_close_alert(self):
        self.post_message(self.CloseAlert())

    @on(CloseAlert)
    @work(exclusive=True, exit_on_error=False, thread=False)
    async def close_alert(self):
        current_status = self._get_cursor_status()
        if str(current_status) == "closed":
            self.notify(
                severity="warning",
                message=f'Alert can not be closed because in status "{current_status}"',
            )
            return

        opsgenie_id = self._get_cursor_opsgenie_id()
        if opsgenie_id is None:
            self._notify_invalid_opsgenie_id()
            return

        message = self._get_cursor_message()
        self.notify(f"Closing alert\n{message}...")
        await opsgenie.client.api.close_alert(
            parameters={"identifier": opsgenie_id},
            note=self._get_note_on_action("close"),
        )
        self.post_message(self.LookupDataWithDelay(delay=1.75))

    def action_post_remove_tag_alert(self):
        self.post_message(self.RemoveTagAlert())

    def _get_cursor_rowkey(self):
        dt = self.query_one("#alerts_data_table", TygenieDataTable)
        row_key, _ = dt.coordinate_to_cell_key(dt.cursor_coordinate)
        return row_key

    def _get_cursor_values(self) -> list:
        row_key = self._get_cursor_rowkey()
        dt = self.query_one("#alerts_data_table", TygenieDataTable)
        return dt.get_row(row_key)

    def _get_cursor_message(self) -> str:
        displayed_fields = [k for k in self.app.alerts_list_formatter.displayed_fields]
        return self._get_cursor_values()[displayed_fields.index("message")]

    def _get_cursor_status(self) -> str:
        displayed_fields = [k for k in self.app.alerts_list_formatter.displayed_fields]
        return self._get_cursor_values()[displayed_fields.index("status")]

    def _get_cursor_opsgenie_id(self, row_key=None) -> str | None:
        try:
            if row_key is None:
                row_key = self._get_cursor_rowkey()

            return re.sub("data_table_", "", str(row_key.value))
        except CellDoesNotExist:
            return None

    @on(AlertDetails.UpdateAlertDetails)
    def _update_alert_details(self, event):
        alert_detail_title = self.query_one("#alert_details_title", AlertDetailTitle)
        alert_detail_title.title = Content.from_markup(
            "$message", message=event.alert.message
        )

        def set_class_handler(css_class):
            classes = ["open", "closed", "acked"]
            for class_to_remove in [c for c in classes if c != css_class]:
                alert_detail_title.remove_class(class_to_remove)
            alert_detail_title.add_class(css_class)

        if event.alert.status == "open":
            if event.alert.acknowledged:
                set_class_handler("acked")
            else:
                set_class_handler("open")

        if event.alert.status == "closed":
            set_class_handler("closed")

        alert_details_details_swicher = self.query_one(
            "#alert_details_details_switcher", ContentSwitcher
        )

        if event.alert.details:
            alert_details = self.query_one("#datatable_alert_details", DataTable)
            alert_details.cursor_type = "row"
            alert_details.zebra_stripes = True
            alert_details.show_header = False
            alert_details.show_cursor = False
            alert_details.cell_padding = 1

            alert_details.clear(columns=True)
            if not alert_details.columns:
                alert_details.add_columns(*["key", "value"])
            for key, value in sorted(event.alert.details.additional_properties.items()):
                alert_details.add_row(*tuple([key, value]))

            # alert_details.update(event.alert.details)
            alert_details_details_swicher.current = "alert_detail_pretty_container"
        else:
            alert_details_details_swicher.current = "no_alert_details_label"

        alert_details_tags_swicher = self.query_one(
            "#alert_details_tags_switcher", ContentSwitcher
        )

        if event.alert.tags:
            alert_tags = self.query_one("#alert_tags_checkboxes", Tags)
            alert_tags.update(names=event.alert.tags)
            alert_details_tags_swicher.current = "alert_tags_container"
        else:
            alert_details_tags_swicher.current = "no_alert_tags_label"

        alert_details_description_swicher = self.query_one(
            "#alert_details_description_switcher", ContentSwitcher
        )

        if event.alert.description:
            alert_description = self.query_one("#pretty_alert_description", Markdown)
            self.app.alert_description_formatter.content = event.alert.description
            self.app.alert_description_formatter.format()

            alert_description.update(self.app.alert_description_formatter.content)
            alert_details_description_swicher.current = "alert_description_md_container"
        else:
            alert_details_description_swicher.current = "no_alert_description_label"

        pretty = self.query_one("#pretty_raw_alert_detail", Pretty)
        pretty.update(event.alert)

    @on(AlertNotes.UpdateAlertNotes)
    def _update_alert_notes(self, event):
        content_switcher = self.query_one("#alert_notes_switcher", ContentSwitcher)
        if len(event.alert_notes):
            md_content = ""
            date_format = config.ty_config.tygenie.get("alerts", {}).get(
                "date_format", "%d/%m %H:%M"
            )
            md = self.query_one("#md_alert_note", Markdown)
            for note in event.alert_notes:
                created_at = dtfstr(note.created_at.isoformat()).strftime(date_format)
                md_content += f"""
- *{created_at}* by **{re.sub('@.*$', '', note.owner)}**

> {note.note}
"""
            md.update(md_content)
            content_switcher.current = "alert_notes_markdown_container"
        else:
            content_switcher.current = "no_note_label"

    def _set_alerts_list_focus(self):
        self.query_one("#alerts_data_table", TygenieDataTable).focus()

    def action_alerts_list_focus(self):
        self._set_alerts_list_focus()

    def _set_alert_details_focus(self):
        self.app.query_one(
            "#alert_detail_tabbed_content", AlertDetailsTabbedContent
        ).focus()

    def action_alert_details_focus(self):
        self._set_alert_details_focus()

    def _set_alert_notes_focus(self):
        self.app.query_one("#alert_notes", AlertNotes).focus()

    def action_alert_notes_focus(self):
        self._set_alert_notes_focus()

    def _notify_invalid_opsgenie_id(self):
        self.notify(message="No valid opsgenie id found", severity="error")

    @on(AlertActionContainer.OpenInBrowser)
    def open_in_webbrowser(self):
        # keep focus on alert list
        self._set_alerts_list_focus()
        opsgenie_id = self._get_cursor_opsgenie_id()
        if opsgenie_id is None:
            self._notify_invalid_opsgenie_id()
            return

        webapp_url = config.ty_config.opsgenie.get("webapp_url")
        webbrowser.open_new_tab(f"{webapp_url}/alert/detail/{opsgenie_id}/details")

    def action_open_in_webbrowser(self):
        self.open_in_webbrowser()

    @on(TagAlert)
    @on(AlertActionContainer.AddTag)
    @work(exclusive=True, exit_on_error=False, thread=False)
    async def tag_alert(self):
        tag_value = self.query_one(
            "#alert_action_container", AlertActionContainer
        ).tag_value

        opsgenie_id = self._get_cursor_opsgenie_id()
        if opsgenie_id is None:
            self._notify_invalid_opsgenie_id()
            return

        message = self._get_cursor_message()
        self.notify(f'Adding tag "{tag_value}" on alert: \n{message}')
        note = f"{self._get_note_on_action('tag')}".format(tag=tag_value)
        await opsgenie.client.api.tag_alert(
            parameters={"identifier": opsgenie_id}, tags=[tag_value], note=note
        )
        self.post_message(self.LookupDataWithDelay(delay=1.75))

    def action_post_tag_alert(self):
        self.post_message(self.TagAlert())

    @on(RemoveTagAlert)
    @work(exclusive=True, exit_on_error=False, thread=False)
    async def remove_default_tag_from_selected_alert(self):
        tag: Input = self.query_one(
            "#alert_action_container", AlertActionContainer
        ).query_one("#tag_input", Input)

        opsgenie_id = self._get_cursor_opsgenie_id()
        if opsgenie_id is None:
            self._notify_invalid_opsgenie_id()
            return

        message = self._get_cursor_message()
        self.notify(f'Removing tag "{tag.value}" on alert: \n{message}')
        note = f"{self._get_note_on_action('untag')}".format(tag=tag.value)
        await opsgenie.client.api.remove_tag_alert(
            parameters={"identifier": opsgenie_id}, tags=[tag.value], note=note
        )
        self.post_message(self.LookupDataWithDelay(delay=1.75))

    @on(Tags.RemoveTag)
    @work(exclusive=True, exit_on_error=False, thread=False)
    async def remove_tag(self, tag):
        opsgenie_id = self._get_cursor_opsgenie_id()
        if opsgenie_id is None:
            self._notify_invalid_opsgenie_id()
            return

        message = self._get_cursor_message()
        self.notify(f'Removing tag "{tag.value}" on alert: \n{message}')

        await opsgenie.client.api.remove_tag_alert(
            parameters={"identifier": opsgenie_id}, tags=[tag.value]
        )
        self.post_message(self.LookupDataWithDelay(delay=1.75))

    @on(AddNote)
    async def _add_note_from_message(self, message: AddNote):
        opsgenie_id: str = message.opsgenie_id
        note: str = message.note
        if not note or not opsgenie_id:
            return
        await opsgenie.client.api.add_note(
            parameters={"identifier": opsgenie_id}, note=note
        )

    @on(AlertActionContainer.AddNote)
    @work(exclusive=True, exit_on_error=False, thread=False)
    async def action_open_add_note(self):
        opsgenie_id = self._get_cursor_opsgenie_id()
        if opsgenie_id is None:
            self._notify_invalid_opsgenie_id()
            return

        async def _add_note(result):
            await opsgenie.client.api.add_note(
                parameters={"identifier": opsgenie_id}, note=result
            )
            if result and not re.match("^\n+$", result):
                self.notify(f"Note added to alert #f{opsgenie_id}")
                self.post_message(self.LookupDataWithDelay(delay=1))

        self.app.push_screen("add_note", _add_note)



import json
from typing import TYPE_CHECKING

from textual.app import ComposeResult
from textual.binding import Binding
from textual.containers import Grid
from textual.message import Message
from textual.widgets import Button, Footer, TextArea

import tygenie.config as config
import tygenie.logger as logger
import tygenie.opsgenie as opsgenie
from tygenie import consts
from tygenie.screen import TyScreen
from tygenie.widgets.textarea import TygenieTextArea

if TYPE_CHECKING:
    from tygenie.app import TygenieApp


class SettingsScreen(TyScreen):

    BINDINGS = [
        Binding(
            key="ctrl+a", action="activate_alerts_screen()", description="Alerts screen"
        ),
        Binding(key="ctrl+s", action="save_config()", description="Save config"),
        Binding(key="ctrl+q", action="quit", description="Quit"),
    ]

    def __init__(self):
        self.original_text = ""
        self.has_changed = True
        super().__init__()
        self.app: "TygenieApp"

    async def on_screen_resume(self):
        await self.recompose()

    class SettingsUpdated(Message):
        """Message to indicate that settings have been updated and that we go back to alert list"""

    def compose(self) -> ComposeResult:
        config.ty_config.load()
        textarea = TygenieTextArea.code_editor(
            json.dumps(config.ty_config.config, indent=2),
            soft_wrap=True,
            language="json",
            id="settings_textarea",
        )
        self.original_text = textarea.text
        yield Grid(
            textarea,
            Button("Save", variant="success", id="save"),
            id="settings_grid",
        )
        yield Footer()

    async def action_save_config(self):
        await self.save_config()

    def action_activate_alerts_screen(self):
        if self.has_changed:
            self.post_message(self.SettingsUpdated())
        self.app.switch_screen(f"{consts.ALERTS_SCREEN_NAME}")

    async def _save_config(self):
        textarea = self.query_one("#settings_textarea", TextArea)
        try:
            original_json = json.loads(self.original_text)
            current_json = json.loads(textarea.text)

            if original_json == current_json:
                self.notify("No change detected")
                self.has_changed = False
            else:
                config.ty_config.save(config=json.loads(textarea.text))
                self.has_changed = True
        except Exception as e:
            self.has_changed = False
            self.notify(severity="error", message=f"Unable to write configuration: {e}")
        else:
            if self.has_changed:
                self.original_text = textarea.text
                self.notify(
                    severity="information", message="Configuration successfully saved"
                )
            try:
                config.ty_config.reload()
                opsgenie.reload()
                logger.reload()

                await self.app.reload()
            except Exception as e:
                self.notify(severity="error", message=f"Failed to relaod app: {e}")

    async def save_config(self):
        await self._save_config()
        if config.ty_config.sample_copied:
            self.notify(f"Sample copied: {config.ty_config.sample_copied}")
            # As it is the first config, go back to alerts list
            self.action_activate_alerts_screen()

    async def on_button_pressed(self, event: Button.Pressed) -> None:
        if event.button.id == "save":
            await self.save_config()



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.update_service_payload import UpdateServicePayload
from ...types import Response


def _get_kwargs(
    identifier: str,
    *,
    body: UpdateServicePayload,
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}

    _kwargs: Dict[str, Any] = {
        "method": "patch",
        "url": f"/v1/services/{identifier}",
    }

    _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json;charset=UTF-8"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[Any]:
    if response.status_code == 200:
        return None
    if response.status_code == 400:
        return None
    if response.status_code == 401:
        return None
    if response.status_code == 402:
        return None
    if response.status_code == 403:
        return None
    if response.status_code == 404:
        return None
    if response.status_code == 422:
        return None
    if response.status_code == 429:
        return None
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    identifier: str,
    *,
    client: Union[AuthenticatedClient, Client],
    body: UpdateServicePayload,
) -> Response[Any]:
    """Update Service (Partial)

     Update service with given id

    Args:
        identifier (str):
        body (UpdateServicePayload):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        identifier=identifier,
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio_detailed(
    identifier: str,
    *,
    client: Union[AuthenticatedClient, Client],
    body: UpdateServicePayload,
) -> Response[Any]:
    """Update Service (Partial)

     Update service with given id

    Args:
        identifier (str):
        body (UpdateServicePayload):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        identifier=identifier,
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)



from textual.binding import Binding
from textual.widgets import TextArea


class TygenieTextArea(TextArea, inherit_bindings=False):
    BINDINGS = [
        Binding("up", "cursor_up", "Cursor up", show=False),
        Binding("down", "cursor_down", "Cursor down", show=False),
        Binding("left", "cursor_left", "Cursor left", show=False),
        Binding("right", "cursor_right", "Cursor right", show=False),
        Binding("ctrl+left", "cursor_word_left", "Cursor word left", show=False),
        Binding("ctrl+right", "cursor_word_right", "Cursor word right", show=False),
        Binding("home", "cursor_line_start", "Cursor line start", show=False),
        Binding("end,ctrl+e", "cursor_line_end", "Cursor line end", show=False),
        Binding("pageup", "cursor_page_up", "Cursor page up", show=False),
        Binding("pagedown", "cursor_page_down", "Cursor page down", show=False),
        # Making selections (generally holding the shift key and moving cursor)
        Binding(
            "ctrl+shift+left",
            "cursor_word_left(True)",
            "Cursor left word select",
            show=False,
        ),
        Binding(
            "ctrl+shift+right",
            "cursor_word_right(True)",
            "Cursor right word select",
            show=False,
        ),
        Binding(
            "shift+home",
            "cursor_line_start(True)",
            "Cursor line start select",
            show=False,
        ),
        Binding(
            "shift+end", "cursor_line_end(True)", "Cursor line end select", show=False
        ),
        Binding("shift+up", "cursor_up(True)", "Cursor up select", show=False),
        Binding("shift+down", "cursor_down(True)", "Cursor down select", show=False),
        Binding("shift+left", "cursor_left(True)", "Cursor left select", show=False),
        Binding("shift+right", "cursor_right(True)", "Cursor right select", show=False),
        # Shortcut ways of making selections
        # Binding("f5", "select_word", "select word", show=False),
        Binding("f6", "select_line", "Select line", show=False),
        Binding("f7", "select_all", "Select all", show=False),
        # Deletion
        Binding("backspace", "delete_left", "Delete character left", show=False),
        Binding(
            "ctrl+w", "delete_word_left", "Delete left to start of word", show=False
        ),
        Binding("delete,ctrl+d", "delete_right", "Delete character right", show=False),
        Binding(
            "ctrl+f", "delete_word_right", "Delete right to start of word", show=False
        ),
        Binding("ctrl+x", "delete_line", "Delete line", show=False),
        Binding(
            "ctrl+u", "delete_to_start_of_line", "Delete to line start", show=False
        ),
        Binding(
            "ctrl+k",
            "delete_to_end_of_line_or_delete_line",
            "Delete to line end",
            show=False,
        ),
        Binding("ctrl+z", "undo", "Undo", show=False),
        Binding("ctrl+y", "redo", "Redo", show=False),
    ]



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.update_user_payload_details import UpdateUserPayloadDetails
    from ..models.user_address import UserAddress
    from ..models.user_role import UserRole


T = TypeVar("T", bound="UpdateUserPayload")


@_attrs_define
class UpdateUserPayload:
    """
    Attributes:
        username (Union[Unset, str]): E-mail address of the user
        full_name (Union[Unset, str]): Name of the user
        role (Union[Unset, UserRole]):
        skype_username (Union[Unset, str]): Skype username of the user
        time_zone (Union[Unset, str]): Timezone of the user. If not set, timezone of the customer will be used instead.
        locale (Union[Unset, str]): Location information of the user. If not set, locale of the customer will be used
            instead.
        user_address (Union[Unset, UserAddress]):
        tags (Union[Unset, List[str]]): List of labels attached to the user. You can label users to differentiate them
            from the rest. For example, you can add ITManager tag to differentiate people with this role from others.
        details (Union[Unset, UpdateUserPayloadDetails]): Set of user defined properties.
        invitation_disabled (Union[Unset, bool]): Invitation email will not be sent if set to true. Default value is
            false
    """

    username: Union[Unset, str] = UNSET
    full_name: Union[Unset, str] = UNSET
    role: Union[Unset, "UserRole"] = UNSET
    skype_username: Union[Unset, str] = UNSET
    time_zone: Union[Unset, str] = UNSET
    locale: Union[Unset, str] = UNSET
    user_address: Union[Unset, "UserAddress"] = UNSET
    tags: Union[Unset, List[str]] = UNSET
    details: Union[Unset, "UpdateUserPayloadDetails"] = UNSET
    invitation_disabled: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        username = self.username

        full_name = self.full_name

        role: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.role, Unset):
            role = self.role.to_dict()

        skype_username = self.skype_username

        time_zone = self.time_zone

        locale = self.locale

        user_address: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.user_address, Unset):
            user_address = self.user_address.to_dict()

        tags: Union[Unset, List[str]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags

        details: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.details, Unset):
            details = self.details.to_dict()

        invitation_disabled = self.invitation_disabled

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if username is not UNSET:
            field_dict["username"] = username
        if full_name is not UNSET:
            field_dict["fullName"] = full_name
        if role is not UNSET:
            field_dict["role"] = role
        if skype_username is not UNSET:
            field_dict["skypeUsername"] = skype_username
        if time_zone is not UNSET:
            field_dict["timeZone"] = time_zone
        if locale is not UNSET:
            field_dict["locale"] = locale
        if user_address is not UNSET:
            field_dict["userAddress"] = user_address
        if tags is not UNSET:
            field_dict["tags"] = tags
        if details is not UNSET:
            field_dict["details"] = details
        if invitation_disabled is not UNSET:
            field_dict["invitationDisabled"] = invitation_disabled

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.update_user_payload_details import UpdateUserPayloadDetails
        from ..models.user_address import UserAddress
        from ..models.user_role import UserRole

        d = src_dict.copy()
        username = d.pop("username", UNSET)

        full_name = d.pop("fullName", UNSET)

        _role = d.pop("role", UNSET)
        role: Union[Unset, UserRole]
        if isinstance(_role, Unset):
            role = UNSET
        else:
            role = UserRole.from_dict(_role)

        skype_username = d.pop("skypeUsername", UNSET)

        time_zone = d.pop("timeZone", UNSET)

        locale = d.pop("locale", UNSET)

        _user_address = d.pop("userAddress", UNSET)
        user_address: Union[Unset, UserAddress]
        if isinstance(_user_address, Unset):
            user_address = UNSET
        else:
            user_address = UserAddress.from_dict(_user_address)

        tags = cast(List[str], d.pop("tags", UNSET))

        _details = d.pop("details", UNSET)
        details: Union[Unset, UpdateUserPayloadDetails]
        if isinstance(_details, Unset):
            details = UNSET
        else:
            details = UpdateUserPayloadDetails.from_dict(_details)

        invitation_disabled = d.pop("invitationDisabled", UNSET)

        update_user_payload = cls(
            username=username,
            full_name=full_name,
            role=role,
            skype_username=skype_username,
            time_zone=time_zone,
            locale=locale,
            user_address=user_address,
            tags=tags,
            details=details,
            invitation_disabled=invitation_disabled,
        )

        update_user_payload.additional_properties = d
        return update_user_payload

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class DurationTimeUnit(str, Enum):
    DAYS = "days"
    HOURS = "hours"
    MICROS = "micros"
    MILISECONDS = "miliseconds"
    MINUTES = "minutes"
    NANOS = "nanos"
    SECONDS = "seconds"

    def __str__(self) -> str:
        return str(self.value)



from textual.binding import Binding
from textual.widgets import TabbedContent, Tabs


class TygenieTabbedContent(TabbedContent):
    BINDINGS = [
        Binding("l", "next_tab", "Next tab", show=False),
        Binding("h", "previous_tab", "Previous tab", show=False),
        Binding("down,j", "app.focus_next", "Focus next", show=False),
        Binding("up,k", "app.focus_previous", "Focus previous", show=False),
    ]

    def action_next_tab(self) -> None:
        tabs = self.query_one(Tabs)
        if tabs.has_focus:
            tabs.action_next_tab()

    def action_previous_tab(self) -> None:
        tabs = self.query_one(Tabs)
        if tabs.has_focus:
            tabs.action_previous_tab()



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="UserAddress")


@_attrs_define
class UserAddress:
    """
    Attributes:
        country (Union[Unset, str]):
        state (Union[Unset, str]):
        city (Union[Unset, str]):
        line (Union[Unset, str]):
        zip_code (Union[Unset, str]):
    """

    country: Union[Unset, str] = UNSET
    state: Union[Unset, str] = UNSET
    city: Union[Unset, str] = UNSET
    line: Union[Unset, str] = UNSET
    zip_code: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        country = self.country

        state = self.state

        city = self.city

        line = self.line

        zip_code = self.zip_code

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if country is not UNSET:
            field_dict["country"] = country
        if state is not UNSET:
            field_dict["state"] = state
        if city is not UNSET:
            field_dict["city"] = city
        if line is not UNSET:
            field_dict["line"] = line
        if zip_code is not UNSET:
            field_dict["zipCode"] = zip_code

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        country = d.pop("country", UNSET)

        state = d.pop("state", UNSET)

        city = d.pop("city", UNSET)

        line = d.pop("line", UNSET)

        zip_code = d.pop("zipCode", UNSET)

        user_address = cls(
            country=country,
            state=state,
            city=city,
            line=line,
            zip_code=zip_code,
        )

        user_address.additional_properties = d
        return user_address

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="AlertReport")


@_attrs_define
class AlertReport:
    """
    Attributes:
        ack_time (Union[Unset, int]):
        close_time (Union[Unset, int]):
        acknowledged_by (Union[Unset, str]):
        closed_by (Union[Unset, str]):
    """

    ack_time: Union[Unset, int] = UNSET
    close_time: Union[Unset, int] = UNSET
    acknowledged_by: Union[Unset, str] = UNSET
    closed_by: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        ack_time = self.ack_time

        close_time = self.close_time

        acknowledged_by = self.acknowledged_by

        closed_by = self.closed_by

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if ack_time is not UNSET:
            field_dict["ackTime"] = ack_time
        if close_time is not UNSET:
            field_dict["closeTime"] = close_time
        if acknowledged_by is not UNSET:
            field_dict["acknowledgedBy"] = acknowledged_by
        if closed_by is not UNSET:
            field_dict["closedBy"] = closed_by

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        ack_time = d.pop("ackTime", UNSET)

        close_time = d.pop("closeTime", UNSET)

        acknowledged_by = d.pop("acknowledgedBy", UNSET)

        closed_by = d.pop("closedBy", UNSET)

        alert_report = cls(
            ack_time=ack_time,
            close_time=close_time,
            acknowledged_by=acknowledged_by,
            closed_by=closed_by,
        )

        alert_report.additional_properties = d
        return alert_report

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.create_heartbeat_payload_alert_priority import CreateHeartbeatPayloadAlertPriority
from ..models.create_heartbeat_payload_interval_unit import CreateHeartbeatPayloadIntervalUnit
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.create_heartbeat_payload_owner_team import CreateHeartbeatPayloadOwnerTeam


T = TypeVar("T", bound="CreateHeartbeatPayload")


@_attrs_define
class CreateHeartbeatPayload:
    """
    Attributes:
        name (str): Name of the heartbeat
        interval (int): Specifies how often a heartbeat message should be expected
        interval_unit (CreateHeartbeatPayloadIntervalUnit): Interval specified as 'minutes', 'hours' or 'days'
        enabled (bool): Enable/disable heartbeat monitoring
        description (Union[Unset, str]): An optional description of the heartbeat
        owner_team (Union[Unset, CreateHeartbeatPayloadOwnerTeam]): Owner team of the heartbeat, consisting id and/or
            name of the owner team
        alert_message (Union[Unset, str]): Specifies the alert message for heartbeat expiration alert. If this is not
            provided, default alert message is 'HeartbeatName is expired'
        alert_tags (Union[Unset, List[str]]): Specifies the alert tags for heartbeat expiration alert
        alert_priority (Union[Unset, CreateHeartbeatPayloadAlertPriority]): Specifies the alert priority for heartbeat
            expiration alert. If this is not provided, default priority is P3
    """

    name: str
    interval: int
    interval_unit: CreateHeartbeatPayloadIntervalUnit
    enabled: bool
    description: Union[Unset, str] = UNSET
    owner_team: Union[Unset, "CreateHeartbeatPayloadOwnerTeam"] = UNSET
    alert_message: Union[Unset, str] = UNSET
    alert_tags: Union[Unset, List[str]] = UNSET
    alert_priority: Union[Unset, CreateHeartbeatPayloadAlertPriority] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        interval = self.interval

        interval_unit = self.interval_unit.value

        enabled = self.enabled

        description = self.description

        owner_team: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.owner_team, Unset):
            owner_team = self.owner_team.to_dict()

        alert_message = self.alert_message

        alert_tags: Union[Unset, List[str]] = UNSET
        if not isinstance(self.alert_tags, Unset):
            alert_tags = self.alert_tags

        alert_priority: Union[Unset, str] = UNSET
        if not isinstance(self.alert_priority, Unset):
            alert_priority = self.alert_priority.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "interval": interval,
                "intervalUnit": interval_unit,
                "enabled": enabled,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description
        if owner_team is not UNSET:
            field_dict["ownerTeam"] = owner_team
        if alert_message is not UNSET:
            field_dict["alertMessage"] = alert_message
        if alert_tags is not UNSET:
            field_dict["alertTags"] = alert_tags
        if alert_priority is not UNSET:
            field_dict["alertPriority"] = alert_priority

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.create_heartbeat_payload_owner_team import CreateHeartbeatPayloadOwnerTeam

        d = src_dict.copy()
        name = d.pop("name")

        interval = d.pop("interval")

        interval_unit = CreateHeartbeatPayloadIntervalUnit(d.pop("intervalUnit"))

        enabled = d.pop("enabled")

        description = d.pop("description", UNSET)

        _owner_team = d.pop("ownerTeam", UNSET)
        owner_team: Union[Unset, CreateHeartbeatPayloadOwnerTeam]
        if isinstance(_owner_team, Unset):
            owner_team = UNSET
        else:
            owner_team = CreateHeartbeatPayloadOwnerTeam.from_dict(_owner_team)

        alert_message = d.pop("alertMessage", UNSET)

        alert_tags = cast(List[str], d.pop("alertTags", UNSET))

        _alert_priority = d.pop("alertPriority", UNSET)
        alert_priority: Union[Unset, CreateHeartbeatPayloadAlertPriority]
        if isinstance(_alert_priority, Unset):
            alert_priority = UNSET
        else:
            alert_priority = CreateHeartbeatPayloadAlertPriority(_alert_priority)

        create_heartbeat_payload = cls(
            name=name,
            interval=interval,
            interval_unit=interval_unit,
            enabled=enabled,
            description=description,
            owner_team=owner_team,
            alert_message=alert_message,
            alert_tags=alert_tags,
            alert_priority=alert_priority,
        )

        create_heartbeat_payload.additional_properties = d
        return create_heartbeat_payload

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.service_audience_template_stakeholder_condition_match_type import (
    ServiceAudienceTemplateStakeholderConditionMatchType,
)
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.service_audience_template_condition import ServiceAudienceTemplateCondition


T = TypeVar("T", bound="ServiceAudienceTemplateStakeholder")


@_attrs_define
class ServiceAudienceTemplateStakeholder:
    """
    Attributes:
        individuals (Union[Unset, List[str]]):
        condition_match_type (Union[Unset, ServiceAudienceTemplateStakeholderConditionMatchType]): Match type for given
            conditions. Possible values are [match-all-conditions, match-any-condition]. Default value is [match-any-
            condition].
        conditions (Union[Unset, List['ServiceAudienceTemplateCondition']]):
    """

    individuals: Union[Unset, List[str]] = UNSET
    condition_match_type: Union[Unset, ServiceAudienceTemplateStakeholderConditionMatchType] = UNSET
    conditions: Union[Unset, List["ServiceAudienceTemplateCondition"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        individuals: Union[Unset, List[str]] = UNSET
        if not isinstance(self.individuals, Unset):
            individuals = self.individuals

        condition_match_type: Union[Unset, str] = UNSET
        if not isinstance(self.condition_match_type, Unset):
            condition_match_type = self.condition_match_type.value

        conditions: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.conditions, Unset):
            conditions = []
            for conditions_item_data in self.conditions:
                conditions_item = conditions_item_data.to_dict()
                conditions.append(conditions_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if individuals is not UNSET:
            field_dict["individuals"] = individuals
        if condition_match_type is not UNSET:
            field_dict["conditionMatchType"] = condition_match_type
        if conditions is not UNSET:
            field_dict["conditions"] = conditions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.service_audience_template_condition import ServiceAudienceTemplateCondition

        d = src_dict.copy()
        individuals = cast(List[str], d.pop("individuals", UNSET))

        _condition_match_type = d.pop("conditionMatchType", UNSET)
        condition_match_type: Union[Unset, ServiceAudienceTemplateStakeholderConditionMatchType]
        if isinstance(_condition_match_type, Unset):
            condition_match_type = UNSET
        else:
            condition_match_type = ServiceAudienceTemplateStakeholderConditionMatchType(_condition_match_type)

        conditions = []
        _conditions = d.pop("conditions", UNSET)
        for conditions_item_data in _conditions or []:
            conditions_item = ServiceAudienceTemplateCondition.from_dict(conditions_item_data)

            conditions.append(conditions_item)

        service_audience_template_stakeholder = cls(
            individuals=individuals,
            condition_match_type=condition_match_type,
            conditions=conditions,
        )

        service_audience_template_stakeholder.additional_properties = d
        return service_audience_template_stakeholder

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.recipient import Recipient
    from ..models.schedule_override_rotation import ScheduleOverrideRotation


T = TypeVar("T", bound="UpdateScheduleOverridePayload")


@_attrs_define
class UpdateScheduleOverridePayload:
    """
    Attributes:
        user (Recipient):
        start_date (datetime.datetime): Time for override starting
        end_date (datetime.datetime): Time for override ending
        rotations (Union[Unset, List['ScheduleOverrideRotation']]): Identifier (id or name) of rotations that override
            will apply. When it's set, only specified schedule rotations will be overridden
    """

    user: "Recipient"
    start_date: datetime.datetime
    end_date: datetime.datetime
    rotations: Union[Unset, List["ScheduleOverrideRotation"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        user = self.user.to_dict()

        start_date = self.start_date.isoformat()

        end_date = self.end_date.isoformat()

        rotations: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.rotations, Unset):
            rotations = []
            for rotations_item_data in self.rotations:
                rotations_item = rotations_item_data.to_dict()
                rotations.append(rotations_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "user": user,
                "startDate": start_date,
                "endDate": end_date,
            }
        )
        if rotations is not UNSET:
            field_dict["rotations"] = rotations

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.recipient import Recipient
        from ..models.schedule_override_rotation import ScheduleOverrideRotation

        d = src_dict.copy()
        user = Recipient.from_dict(d.pop("user"))

        start_date = isoparse(d.pop("startDate"))

        end_date = isoparse(d.pop("endDate"))

        rotations = []
        _rotations = d.pop("rotations", UNSET)
        for rotations_item_data in _rotations or []:
            rotations_item = ScheduleOverrideRotation.from_dict(rotations_item_data)

            rotations.append(rotations_item)

        update_schedule_override_payload = cls(
            user=user,
            start_date=start_date,
            end_date=end_date,
            rotations=rotations,
        )

        update_schedule_override_payload.additional_properties = d
        return update_schedule_override_payload

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Contains all the data models used in inputs/outputs"""

from .account_info import AccountInfo
from .account_plan import AccountPlan
from .acknowledge_alert_identifier_type import AcknowledgeAlertIdentifierType
from .action_categorized import ActionCategorized
from .action_mapping import ActionMapping
from .action_mapping_action import ActionMappingAction
from .action_mapping_mapped_action import ActionMappingMappedAction
from .add_attachment_alert_identifier_type import AddAttachmentAlertIdentifierType
from .add_attachment_body import AddAttachmentBody
from .add_details_identifier_type import AddDetailsIdentifierType
from .add_details_to_alert_payload import AddDetailsToAlertPayload
from .add_details_to_alert_payload_details import AddDetailsToAlertPayloadDetails
from .add_note_identifier_type import AddNoteIdentifierType
from .add_responder_identifier_type import AddResponderIdentifierType
from .add_responder_to_alert_payload import AddResponderToAlertPayload
from .add_tags_identifier_type import AddTagsIdentifierType
from .add_tags_to_alert_payload import AddTagsToAlertPayload
from .add_team_identifier_type import AddTeamIdentifierType
from .add_team_member_payload import AddTeamMemberPayload
from .add_team_member_team_identifier_type import AddTeamMemberTeamIdentifierType
from .add_team_to_alert_payload import AddTeamToAlertPayload
from .airbrake_integration import AirbrakeIntegration
from .alert import Alert
from .alert_action_payload import AlertActionPayload
from .alert_attachment import AlertAttachment
from .alert_attachment_meta import AlertAttachmentMeta
from .alert_details import AlertDetails
from .alert_filter import AlertFilter
from .alert_filter_condition_match_type import AlertFilterConditionMatchType
from .alert_integration import AlertIntegration
from .alert_log import AlertLog
from .alert_logic_integration import AlertLogicIntegration
from .alert_note import AlertNote
from .alert_paging import AlertPaging
from .alert_policy import AlertPolicy
from .alert_policy_details import AlertPolicyDetails
from .alert_policy_priority import AlertPolicyPriority
from .alert_recipient import AlertRecipient
from .alert_report import AlertReport
from .alert_request_status import AlertRequestStatus
from .alert_site_integration import AlertSiteIntegration
from .alert_team_meta import AlertTeamMeta
from .alert_user_meta import AlertUserMeta
from .amazon_cloud_trail_integration import AmazonCloudTrailIntegration
from .amazon_ec_2_auto_scaling_integration import AmazonEc2AutoScalingIntegration
from .amazon_rds_integration import AmazonRdsIntegration
from .amazon_route_53_health_check_integration import AmazonRoute53HealthCheckIntegration
from .amazon_security_hub_integration import AmazonSecurityHubIntegration
from .amazon_ses_integration import AmazonSesIntegration
from .amazon_sns_callback import AmazonSnsCallback
from .amazon_sns_integration import AmazonSnsIntegration
from .amazon_sns_outgoing_integration import AmazonSnsOutgoingIntegration
from .api_integration import ApiIntegration
from .apica_integration import ApicaIntegration
from .apimetrics_integration import ApimetricsIntegration
from .app_dynamics_integration import AppDynamicsIntegration
from .app_optics_integration import AppOpticsIntegration
from .app_signal_integration import AppSignalIntegration
from .app_signal_v2_integration import AppSignalV2Integration
from .assign_alert_identifier_type import AssignAlertIdentifierType
from .assign_alert_payload import AssignAlertPayload
from .atatus_integration import AtatusIntegration
from .atlassian_bamboo_integration import AtlassianBambooIntegration
from .authenticate_integration_payload import AuthenticateIntegrationPayload
from .authenticate_integration_payload_type import AuthenticateIntegrationPayloadType
from .auto_close_action import AutoCloseAction
from .auto_restart_action import AutoRestartAction
from .auto_task_email_integration import AutoTaskEmailIntegration
from .autotask_aem_email_integration import AutotaskAEMEmailIntegration
from .azure_auto_scale_integration import AzureAutoScaleIntegration
from .azure_integration import AzureIntegration
from .azure_oms_integration import AzureOMSIntegration
from .azure_resource_health_integration import AzureResourceHealthIntegration
from .azure_service_health_integration import AzureServiceHealthIntegration
from .base_alert import BaseAlert
from .base_incident import BaseIncident
from .base_incident_extra_properties import BaseIncidentExtraProperties
from .base_incoming_feature import BaseIncomingFeature
from .base_incoming_feature_extra_properties import BaseIncomingFeatureExtraProperties
from .base_incoming_feature_feature_type import BaseIncomingFeatureFeatureType
from .base_integration_action import BaseIntegrationAction
from .base_integration_action_type import BaseIntegrationActionType
from .base_response import BaseResponse
from .base_response_with_expandable import BaseResponseWithExpandable
from .base_webhook_callback_base_webhook_callback_type import BaseWebhookCallbackBaseWebhookCallbackType
from .bidirectional_callback import BidirectionalCallback
from .bidirectional_callback_bidirectional_callback_type import BidirectionalCallbackBidirectionalCallbackType
from .bidirectional_callback_new import BidirectionalCallbackNew
from .bidirectional_callback_new_bidirectional_callback_type import BidirectionalCallbackNewBidirectionalCallbackType
from .big_panda_integration import BigPandaIntegration
from .bitbucket_integration import BitbucketIntegration
from .blue_matador_integration import BlueMatadorIntegration
from .bmc_foot_prints_v11_callback import BMCFootPrintsV11Callback
from .bmc_foot_prints_v11_integration import BMCFootPrintsV11Integration
from .bmc_foot_prints_v12_callback import BMCFootPrintsV12Callback
from .bmc_foot_prints_v12_integration import BMCFootPrintsV12Integration
from .bmc_remedy_callback import BMCRemedyCallback
from .bmc_remedy_force_integration import BMCRemedyForceIntegration
from .bmc_remedy_integration import BMCRemedyIntegration
from .bmc_remedy_on_demand_callback import BMCRemedyOnDemandCallback
from .bmc_remedy_on_demand_integration import BMCRemedyOnDemandIntegration
from .boundary_integration import BoundaryIntegration
from .callback_condition import CallbackCondition
from .callback_condition_field import CallbackConditionField
from .callback_condition_operation import CallbackConditionOperation
from .campfire_callback import CampfireCallback
from .campfire_integration import CampfireIntegration
from .catchpoint_integration import CatchpointIntegration
from .change_notification_rule_order_payload import ChangeNotificationRuleOrderPayload
from .change_policy_order_payload import ChangePolicyOrderPayload
from .change_team_routing_rule_order_payload import ChangeTeamRoutingRuleOrderPayload
from .change_team_routing_rule_order_team_identifier_type import ChangeTeamRoutingRuleOrderTeamIdentifierType
from .check_mk_integration import CheckMKIntegration
from .cherwell_callback import CherwellCallback
from .cherwell_integration import CherwellIntegration
from .circle_ci_integration import CircleCiIntegration
from .circonus_callback import CirconusCallback
from .circonus_integration import CirconusIntegration
from .close_alert_identifier_type import CloseAlertIdentifierType
from .close_incident_identifier_type import CloseIncidentIdentifierType
from .cloud_monix_integration import CloudMonixIntegration
from .cloud_sploit_integration import CloudSploitIntegration
from .cloud_watch_events_integration import CloudWatchEventsIntegration
from .cloud_watch_integration import CloudWatchIntegration
from .codeship_integration import CodeshipIntegration
from .common_integration_action import CommonIntegrationAction
from .compose_integration import ComposeIntegration
from .condition import Condition
from .condition_field import ConditionField
from .condition_operation import ConditionOperation
from .connect_wise_automate_integration import ConnectWiseAutomateIntegration
from .connect_wise_callback import ConnectWiseCallback
from .connect_wise_integration import ConnectWiseIntegration
from .connect_wise_manage_callback import ConnectWiseManageCallback
from .connect_wise_manage_integration import ConnectWiseManageIntegration
from .connect_wise_manage_v2_callback import ConnectWiseManageV2Callback
from .connect_wise_manage_v2_integration import ConnectWiseManageV2Integration
from .consul_integration import ConsulIntegration
from .contact import Contact
from .contact_meta import ContactMeta
from .contact_meta_method import ContactMetaMethod
from .contact_status import ContactStatus
from .contact_with_apply_order import ContactWithApplyOrder
from .copper_egg_integration import CopperEggIntegration
from .count_alerts_search_identifier_type import CountAlertsSearchIdentifierType
from .crashlytics_integration import CrashlyticsIntegration
from .create_alert_payload import CreateAlertPayload
from .create_alert_payload_details import CreateAlertPayloadDetails
from .create_alert_payload_priority import CreateAlertPayloadPriority
from .create_contact_payload import CreateContactPayload
from .create_contact_payload_method import CreateContactPayloadMethod
from .create_custom_user_role_payload import CreateCustomUserRolePayload
from .create_escalation_payload import CreateEscalationPayload
from .create_forwarding_rule_payload import CreateForwardingRulePayload
from .create_forwarding_rule_response import CreateForwardingRuleResponse
from .create_heartbeat_payload import CreateHeartbeatPayload
from .create_heartbeat_payload_alert_priority import CreateHeartbeatPayloadAlertPriority
from .create_heartbeat_payload_interval_unit import CreateHeartbeatPayloadIntervalUnit
from .create_heartbeat_payload_owner_team import CreateHeartbeatPayloadOwnerTeam
from .create_heartbeat_response import CreateHeartbeatResponse
from .create_incident_payload import CreateIncidentPayload
from .create_incident_payload_details import CreateIncidentPayloadDetails
from .create_incident_payload_priority import CreateIncidentPayloadPriority
from .create_incident_payload_status_page_entry import CreateIncidentPayloadStatusPageEntry
from .create_integration_action import CreateIntegrationAction
from .create_integration_action_extra_properties import CreateIntegrationActionExtraProperties
from .create_integration_action_priority import CreateIntegrationActionPriority
from .create_integration_actions_response import CreateIntegrationActionsResponse
from .create_integration_response import CreateIntegrationResponse
from .create_maintenance_payload import CreateMaintenancePayload
from .create_maintenance_response import CreateMaintenanceResponse
from .create_notification_rule_payload import CreateNotificationRulePayload
from .create_notification_rule_response import CreateNotificationRuleResponse
from .create_notification_rule_step_payload import CreateNotificationRuleStepPayload
from .create_policy_response import CreatePolicyResponse
from .create_saved_search_payload import CreateSavedSearchPayload
from .create_saved_search_response import CreateSavedSearchResponse
from .create_schedule_override_payload import CreateScheduleOverridePayload
from .create_schedule_override_response import CreateScheduleOverrideResponse
from .create_schedule_override_schedule_identifier_type import CreateScheduleOverrideScheduleIdentifierType
from .create_schedule_payload import CreateSchedulePayload
from .create_schedule_response import CreateScheduleResponse
from .create_schedule_rotation_payload import CreateScheduleRotationPayload
from .create_schedule_rotation_payload_type import CreateScheduleRotationPayloadType
from .create_schedule_rotation_schedule_identifier_type import CreateScheduleRotationScheduleIdentifierType
from .create_service_incident_rule_payload import CreateServiceIncidentRulePayload
from .create_service_incident_rule_payload_condition_match_type import (
    CreateServiceIncidentRulePayloadConditionMatchType,
)
from .create_service_payload import CreateServicePayload
from .create_service_payload_visibility import CreateServicePayloadVisibility
from .create_team_payload import CreateTeamPayload
from .create_team_role_payload import CreateTeamRolePayload
from .create_team_role_team_identifier_type import CreateTeamRoleTeamIdentifierType
from .create_team_routing_rule_payload import CreateTeamRoutingRulePayload
from .create_team_routing_rule_team_identifier_type import CreateTeamRoutingRuleTeamIdentifierType
from .create_user_payload import CreateUserPayload
from .create_user_payload_details import CreateUserPayloadDetails
from .custom_user_role import CustomUserRole
from .datadog_callback import DatadogCallback
from .datadog_integration import DatadogIntegration
from .dataloop_io_integration import DataloopIOIntegration
from .deduplication_action import DeduplicationAction
from .deduplication_action_deduplication_action_type import DeduplicationActionDeduplicationActionType
from .delay_action import DelayAction
from .delay_action_delay_option import DelayActionDelayOption
from .delete_alert_identifier_type import DeleteAlertIdentifierType
from .delete_custom_user_role_identifier_type import DeleteCustomUserRoleIdentifierType
from .delete_escalation_identifier_type import DeleteEscalationIdentifierType
from .delete_forwarding_rule_identifier_type import DeleteForwardingRuleIdentifierType
from .delete_incident_identifier_type import DeleteIncidentIdentifierType
from .delete_saved_search_identifier_type import DeleteSavedSearchIdentifierType
from .delete_schedule_identifier_type import DeleteScheduleIdentifierType
from .delete_schedule_override_schedule_identifier_type import DeleteScheduleOverrideScheduleIdentifierType
from .delete_schedule_rotation_schedule_identifier_type import DeleteScheduleRotationScheduleIdentifierType
from .delete_team_identifier_type import DeleteTeamIdentifierType
from .delete_team_member_team_identifier_type import DeleteTeamMemberTeamIdentifierType
from .delete_team_role_identifier_type import DeleteTeamRoleIdentifierType
from .delete_team_role_team_identifier_type import DeleteTeamRoleTeamIdentifierType
from .delete_team_routing_rule_team_identifier_type import DeleteTeamRoutingRuleTeamIdentifierType
from .deprecated_alert_policy import DeprecatedAlertPolicy
from .deprecated_alert_policy_meta import DeprecatedAlertPolicyMeta
from .deprecated_alert_policy_meta_type import DeprecatedAlertPolicyMetaType
from .deprecated_alert_policy_type import DeprecatedAlertPolicyType
from .deprecated_auto_close_alert_policy import DeprecatedAutoCloseAlertPolicy
from .deprecated_auto_restart_notifications_alert_policy import DeprecatedAutoRestartNotificationsAlertPolicy
from .deprecated_change_alert_policy_order_payload import DeprecatedChangeAlertPolicyOrderPayload
from .deprecated_create_alert_policy_response import DeprecatedCreateAlertPolicyResponse
from .deprecated_get_alert_policy_response import DeprecatedGetAlertPolicyResponse
from .deprecated_list_alert_policies_response import DeprecatedListAlertPoliciesResponse
from .deprecated_modify_alert_policy import DeprecatedModifyAlertPolicy
from .deprecated_modify_alert_policy_details import DeprecatedModifyAlertPolicyDetails
from .deprecated_modify_alert_policy_priority import DeprecatedModifyAlertPolicyPriority
from .deprecated_notification_deduplication_alert_policy import DeprecatedNotificationDeduplicationAlertPolicy
from .deprecated_notification_deduplication_alert_policy_deduplication_action_type import (
    DeprecatedNotificationDeduplicationAlertPolicyDeduplicationActionType,
)
from .deprecated_notification_delay_alert_policy import DeprecatedNotificationDelayAlertPolicy
from .deprecated_notification_delay_alert_policy_delay_option import DeprecatedNotificationDelayAlertPolicyDelayOption
from .desk_callback import DeskCallback
from .desk_integration import DeskIntegration
from .detectify_integration import DetectifyIntegration
from .disable_heartbeat_response import DisableHeartbeatResponse
from .disable_integration_response import DisableIntegrationResponse
from .dns_check_integration import DnsCheckIntegration
from .drip_stat_integration import DripStatIntegration
from .duration import Duration
from .duration_time_unit import DurationTimeUnit
from .dynatrace_app_mon_callback import DynatraceAppMonCallback
from .dynatrace_app_mon_integration import DynatraceAppMonIntegration
from .dynatrace_integration import DynatraceIntegration
from .dynatrace_v2_integration import DynatraceV2Integration
from .email_based_incoming_feature import EmailBasedIncomingFeature
from .email_integration import EmailIntegration
from .enable_heartbeat_response import EnableHeartbeatResponse
from .enable_integration_response import EnableIntegrationResponse
from .error_response import ErrorResponse
from .error_response_response_headers import ErrorResponseResponseHeaders
from .errorception_integration import ErrorceptionIntegration
from .escalate_alert_identifier_type import EscalateAlertIdentifierType
from .escalate_alert_to_next_payload import EscalateAlertToNextPayload
from .escalation import Escalation
from .escalation_recipient import EscalationRecipient
from .escalation_repeat import EscalationRepeat
from .escalation_rule import EscalationRule
from .escalation_rule_condition import EscalationRuleCondition
from .escalation_rule_notify_type import EscalationRuleNotifyType
from .evident_io_integration import EvidentIOIntegration
from .execute_custom_alert_action_identifier_type import ExecuteCustomAlertActionIdentifierType
from .export_schedule_identifier_type import ExportScheduleIdentifierType
from .filter_ import Filter
from .filter_type import FilterType
from .flowdock_callback import FlowdockCallback
from .flowdock_integration import FlowdockIntegration
from .flowdock_v2_callback import FlowdockV2Callback
from .flowdock_v2_integration import FlowdockV2Integration
from .forwarding_rule import ForwardingRule
from .forwarding_rule_meta import ForwardingRuleMeta
from .freshdesk_callback import FreshdeskCallback
from .freshdesk_integration import FreshdeskIntegration
from .freshservice_callback import FreshserviceCallback
from .freshservice_integration import FreshserviceIntegration
from .get_account_info_response import GetAccountInfoResponse
from .get_alert_attachment_response import GetAlertAttachmentResponse
from .get_alert_identifier_type import GetAlertIdentifierType
from .get_alert_response import GetAlertResponse
from .get_attachment_alert_identifier_type import GetAttachmentAlertIdentifierType
from .get_contact_response import GetContactResponse
from .get_count_alerts_response import GetCountAlertsResponse
from .get_count_alerts_response_data import GetCountAlertsResponseData
from .get_custom_user_role_identifier_type import GetCustomUserRoleIdentifierType
from .get_custom_user_role_response import GetCustomUserRoleResponse
from .get_escalation_identifier_type import GetEscalationIdentifierType
from .get_escalation_response import GetEscalationResponse
from .get_forwarding_rule_identifier_type import GetForwardingRuleIdentifierType
from .get_forwarding_rule_response import GetForwardingRuleResponse
from .get_heartbeat_response import GetHeartbeatResponse
from .get_incident_identifier_type import GetIncidentIdentifierType
from .get_incident_request_status_response import GetIncidentRequestStatusResponse
from .get_incident_response import GetIncidentResponse
from .get_integration_response import GetIntegrationResponse
from .get_maintenance_response import GetMaintenanceResponse
from .get_next_on_call_response import GetNextOnCallResponse
from .get_next_on_calls_schedule_identifier_type import GetNextOnCallsScheduleIdentifierType
from .get_notification_rule_response import GetNotificationRuleResponse
from .get_notification_rule_step_response import GetNotificationRuleStepResponse
from .get_on_call_response import GetOnCallResponse
from .get_on_calls_schedule_identifier_type import GetOnCallsScheduleIdentifierType
from .get_policy_response import GetPolicyResponse
from .get_request_status_response import GetRequestStatusResponse
from .get_saved_search_identifier_type import GetSavedSearchIdentifierType
from .get_saved_search_response import GetSavedSearchResponse
from .get_schedule_identifier_type import GetScheduleIdentifierType
from .get_schedule_override_response import GetScheduleOverrideResponse
from .get_schedule_override_schedule_identifier_type import GetScheduleOverrideScheduleIdentifierType
from .get_schedule_response import GetScheduleResponse
from .get_schedule_rotation_response import GetScheduleRotationResponse
from .get_schedule_rotation_schedule_identifier_type import GetScheduleRotationScheduleIdentifierType
from .get_schedule_timeline_identifier_type import GetScheduleTimelineIdentifierType
from .get_schedule_timeline_interval_unit import GetScheduleTimelineIntervalUnit
from .get_schedule_timeline_response import GetScheduleTimelineResponse
from .get_service_audience_template_response import GetServiceAudienceTemplateResponse
from .get_service_incident_rule_response import GetServiceIncidentRuleResponse
from .get_service_response import GetServiceResponse
from .get_team_identifier_type import GetTeamIdentifierType
from .get_team_response import GetTeamResponse
from .get_team_role_identifier_type import GetTeamRoleIdentifierType
from .get_team_role_response import GetTeamRoleResponse
from .get_team_role_team_identifier_type import GetTeamRoleTeamIdentifierType
from .get_team_routing_rule_response import GetTeamRoutingRuleResponse
from .get_team_routing_rule_team_identifier_type import GetTeamRoutingRuleTeamIdentifierType
from .get_user_response import GetUserResponse
from .ghost_inspector_integration import GhostInspectorIntegration
from .git_hub_integration import GitHubIntegration
from .git_lab_integration import GitLabIntegration
from .google_stack_driver_integration import GoogleStackDriverIntegration
from .grafana_integration import GrafanaIntegration
from .grafana_v2_integration import GrafanaV2Integration
from .graylog_integration import GraylogIntegration
from .group_recipient import GroupRecipient
from .heartbeat import Heartbeat
from .heartbeat_integration import HeartbeatIntegration
from .heartbeat_interval_unit import HeartbeatIntervalUnit
from .heartbeat_meta import HeartbeatMeta
from .hip_chat_add_on_callback import HipChatAddOnCallback
from .hip_chat_add_on_integration import HipChatAddOnIntegration
from .hip_chat_callback_v2 import HipChatCallbackV2
from .hip_chat_integration_v2 import HipChatIntegrationV2
from .honeybadger_integration import HoneybadgerIntegration
from .hosted_graphite_integration import HostedGraphiteIntegration
from .hp_service_manager_callback import HPServiceManagerCallback
from .hp_service_manager_integration import HPServiceManagerIntegration
from .humio_integration import HumioIntegration
from .icinga_2_callback import Icinga2Callback
from .icinga_2_integration import Icinga2Integration
from .icinga_callback import IcingaCallback
from .icinga_integration import IcingaIntegration
from .incident import Incident
from .incident_action_payload import IncidentActionPayload
from .incident_request_status import IncidentRequestStatus
from .instana_integration import InstanaIntegration
from .integration import Integration
from .integration_action_filter import IntegrationActionFilter
from .integration_action_filter_condition_match_type import IntegrationActionFilterConditionMatchType
from .integration_condition import IntegrationCondition
from .integration_condition_operation import IntegrationConditionOperation
from .integration_meta import IntegrationMeta
from .integration_type import IntegrationType
from .jenkins_integration import JenkinsIntegration
from .jira_callback import JiraCallback
from .jira_integration import JiraIntegration
from .jira_service_desk_callback import JiraServiceDeskCallback
from .jira_service_desk_integration import JiraServiceDeskIntegration
from .kapacitor_integration import KapacitorIntegration
from .kayako_callback import KayakoCallback
from .kayako_integration import KayakoIntegration
from .lab_tech_email_integration import LabTechEmailIntegration
from .librato_integration import LibratoIntegration
from .libre_nms_callback import LibreNMSCallback
from .libre_nms_integration import LibreNMSIntegration
from .lightstep_integration import LightstepIntegration
from .list_alert_attachments_response import ListAlertAttachmentsResponse
from .list_alert_logs_response import ListAlertLogsResponse
from .list_alert_notes_response import ListAlertNotesResponse
from .list_alert_recipients_response import ListAlertRecipientsResponse
from .list_alerts_order import ListAlertsOrder
from .list_alerts_response import ListAlertsResponse
from .list_alerts_search_identifier_type import ListAlertsSearchIdentifierType
from .list_alerts_sort import ListAlertsSort
from .list_attachments_alert_identifier_type import ListAttachmentsAlertIdentifierType
from .list_contacts_response import ListContactsResponse
from .list_custom_user_roles_response import ListCustomUserRolesResponse
from .list_escalations_response import ListEscalationsResponse
from .list_forwarding_rules_response import ListForwardingRulesResponse
from .list_heartbeat_response import ListHeartbeatResponse
from .list_heartbeat_response_data import ListHeartbeatResponseData
from .list_incidents_order import ListIncidentsOrder
from .list_incidents_response import ListIncidentsResponse
from .list_incidents_sort import ListIncidentsSort
from .list_integration_actions_response import ListIntegrationActionsResponse
from .list_integrations_response import ListIntegrationsResponse
from .list_logs_direction import ListLogsDirection
from .list_logs_identifier_type import ListLogsIdentifierType
from .list_logs_order import ListLogsOrder
from .list_maintenance_response import ListMaintenanceResponse
from .list_maintenance_type import ListMaintenanceType
from .list_notes_direction import ListNotesDirection
from .list_notes_identifier_type import ListNotesIdentifierType
from .list_notes_order import ListNotesOrder
from .list_notification_rule_steps_response import ListNotificationRuleStepsResponse
from .list_notification_rules_response import ListNotificationRulesResponse
from .list_on_calls_response import ListOnCallsResponse
from .list_policies_response import ListPoliciesResponse
from .list_recipients_identifier_type import ListRecipientsIdentifierType
from .list_saved_searches_response import ListSavedSearchesResponse
from .list_schedule_override_response import ListScheduleOverrideResponse
from .list_schedule_override_schedule_identifier_type import ListScheduleOverrideScheduleIdentifierType
from .list_schedule_rotations_response import ListScheduleRotationsResponse
from .list_schedule_rotations_schedule_identifier_type import ListScheduleRotationsScheduleIdentifierType
from .list_schedules_response import ListSchedulesResponse
from .list_service_incident_rules_response import ListServiceIncidentRulesResponse
from .list_services_order import ListServicesOrder
from .list_services_response import ListServicesResponse
from .list_team_logs_identifier_type import ListTeamLogsIdentifierType
from .list_team_logs_order import ListTeamLogsOrder
from .list_team_logs_response import ListTeamLogsResponse
from .list_team_role_response import ListTeamRoleResponse
from .list_team_roles_team_identifier_type import ListTeamRolesTeamIdentifierType
from .list_team_routing_rules_response import ListTeamRoutingRulesResponse
from .list_team_routing_rules_team_identifier_type import ListTeamRoutingRulesTeamIdentifierType
from .list_teams_response import ListTeamsResponse
from .list_user_escalations_response import ListUserEscalationsResponse
from .list_user_forwarding_rules_response import ListUserForwardingRulesResponse
from .list_user_schedules_response import ListUserSchedulesResponse
from .list_user_teams_response import ListUserTeamsResponse
from .list_users_order import ListUsersOrder
from .list_users_response import ListUsersResponse
from .logentries_integration import LogentriesIntegration
from .loggly_integration import LogglyIntegration
from .logic_monitor_callback import LogicMonitorCallback
from .logic_monitor_integration import LogicMonitorIntegration
from .logstash_integration import LogstashIntegration
from .logz_io_integration import LogzIOIntegration
from .looker_integration import LookerIntegration
from .loom_integration import LoomIntegration
from .magentrix_callback import MagentrixCallback
from .magentrix_integration import MagentrixIntegration
from .maintenance import Maintenance
from .maintenance_entity import MaintenanceEntity
from .maintenance_entity_type import MaintenanceEntityType
from .maintenance_meta import MaintenanceMeta
from .maintenance_meta_status import MaintenanceMetaStatus
from .maintenance_rule import MaintenanceRule
from .maintenance_rule_state import MaintenanceRuleState
from .maintenance_status import MaintenanceStatus
from .maintenance_time import MaintenanceTime
from .maintenance_time_type import MaintenanceTimeType
from .manage_engine_integration_dto import ManageEngineIntegrationDTO
from .marid_integration import MaridIntegration
from .match_all_conditions import MatchAllConditions
from .match_any_condition import MatchAnyCondition
from .mattermost_callback import MattermostCallback
from .mattermost_integration import MattermostIntegration
from .mongo_db_cloud_integration import MongoDBCloudIntegration
from .monitis_email_integration import MonitisEmailIntegration
from .monitis_integration import MonitisIntegration
from .ms_teams_callback import MSTeamsCallback
from .ms_teams_integration import MSTeamsIntegration
from .ms_teams_v2_callback import MSTeamsV2Callback
from .ms_teams_v2_integration import MSTeamsV2Integration
from .nagios_based_v1_callback import NagiosBasedV1Callback
from .nagios_based_v2_callback import NagiosBasedV2Callback
from .nagios_integration_v1 import NagiosIntegrationV1
from .nagios_integration_v2 import NagiosIntegrationV2
from .nagios_xi_integration_v1 import NagiosXIIntegrationV1
from .nagios_xi_integration_v2 import NagiosXIIntegrationV2
from .nagios_xiv1_callback import NagiosXIV1Callback
from .nagios_xiv2_callback import NagiosXIV2Callback
from .netuitive_integration import NetuitiveIntegration
from .neustar_integration import NeustarIntegration
from .new_relic_integration import NewRelicIntegration
from .new_relic_v2_integration import NewRelicV2Integration
from .next_on_call import NextOnCall
from .node_ping_integration import NodePingIntegration
from .notification_action_type import NotificationActionType
from .notification_policy import NotificationPolicy
from .notification_repeat import NotificationRepeat
from .notification_rule import NotificationRule
from .notification_rule_meta import NotificationRuleMeta
from .notification_rule_step import NotificationRuleStep
from .notification_rule_step_parent import NotificationRuleStepParent
from .notify_time import NotifyTime
from .observium_integration import ObserviumIntegration
from .observium_v2_integration import ObserviumV2Integration
from .oec_integration import OECIntegration
from .oem_email_integration import OEMEmailIntegration
from .oem_integration import OEMIntegration
from .on_call import OnCall
from .op5_callback import OP5Callback
from .op5_integration import OP5Integration
from .ops_dash_integration import OpsDashIntegration
from .ops_genie_callback import OpsGenieCallback
from .ops_genie_integration import OpsGenieIntegration
from .opsview_integration import OpsviewIntegration
from .outgoing_callback import OutgoingCallback
from .outgoing_callback_callback_type import OutgoingCallbackCallbackType
from .outgoing_callback_new import OutgoingCallbackNew
from .outgoing_callback_new_callback_type import OutgoingCallbackNewCallbackType
from .page_details import PageDetails
from .pager_duty_compatibility_integration import PagerDutyCompatibilityIntegration
from .panopta_integration import PanoptaIntegration
from .papertrail_integration import PapertrailIntegration
from .partial_update_service_audience_template_payload import PartialUpdateServiceAudienceTemplatePayload
from .participant import Participant
from .pingdom_integration import PingdomIntegration
from .pingdom_v2_integration import PingdomV2Integration
from .pingdom_webhook_integration import PingdomWebhookIntegration
from .pingometer_integration import PingometerIntegration
from .planio_callback import PlanioCallback
from .planio_integration import PlanioIntegration
from .policy import Policy
from .policy_meta import PolicyMeta
from .policy_meta_type import PolicyMetaType
from .policy_type import PolicyType
from .prometheus_integration import PrometheusIntegration
from .prtg_callback import PrtgCallback
from .prtg_integration import PrtgIntegration
from .rackspace_integration import RackspaceIntegration
from .raygun_integration import RaygunIntegration
from .recipient import Recipient
from .recipient_type import RecipientType
from .red_gate_sql_monitor_email_integration import RedGateSqlMonitorEmailIntegration
from .remove_attachment_alert_identifier_type import RemoveAttachmentAlertIdentifierType
from .remove_details_identifier_type import RemoveDetailsIdentifierType
from .remove_tags_identifier_type import RemoveTagsIdentifierType
from .responder import Responder
from .responder_type import ResponderType
from .riemann_integration import RiemannIntegration
from .rigor_integration import RigorIntegration
from .ring_central_email_integration import RingCentralEmailIntegration
from .rollbar_callback import RollbarCallback
from .rollbar_integration import RollbarIntegration
from .runscope_integration import RunscopeIntegration
from .sales_force_service_cloud_callback import SalesForceServiceCloudCallback
from .sales_force_service_cloud_integration import SalesForceServiceCloudIntegration
from .salt_stack_integration import SaltStackIntegration
from .saved_search import SavedSearch
from .saved_search_entity import SavedSearchEntity
from .saved_search_meta import SavedSearchMeta
from .scalyr_integration import ScalyrIntegration
from .schedule import Schedule
from .schedule_meta import ScheduleMeta
from .schedule_override import ScheduleOverride
from .schedule_override_meta import ScheduleOverrideMeta
from .schedule_override_rotation import ScheduleOverrideRotation
from .schedule_recipient import ScheduleRecipient
from .schedule_rotation import ScheduleRotation
from .schedule_rotation_type import ScheduleRotationType
from .schedule_rotation_with_parent import ScheduleRotationWithParent
from .schedule_timeline import ScheduleTimeline
from .scom_integration import ScomIntegration
from .scout_integration import ScoutIntegration
from .sematext_spm_integration import SematextSpmIntegration
from .sensu_integration import SensuIntegration
from .sentry_integration import SentryIntegration
from .server_density_integration import ServerDensityIntegration
from .server_guard_24_integration import ServerGuard24Integration
from .service import Service
from .service_audience_template import ServiceAudienceTemplate
from .service_audience_template_condition import ServiceAudienceTemplateCondition
from .service_audience_template_condition_match_field import ServiceAudienceTemplateConditionMatchField
from .service_audience_template_responder import ServiceAudienceTemplateResponder
from .service_audience_template_stakeholder import ServiceAudienceTemplateStakeholder
from .service_audience_template_stakeholder_condition_match_type import (
    ServiceAudienceTemplateStakeholderConditionMatchType,
)
from .service_incident_condition import ServiceIncidentCondition
from .service_incident_condition_field import ServiceIncidentConditionField
from .service_incident_condition_operation import ServiceIncidentConditionOperation
from .service_incident_properties import ServiceIncidentProperties
from .service_incident_properties_details import ServiceIncidentPropertiesDetails
from .service_incident_properties_priority import ServiceIncidentPropertiesPriority
from .service_incident_rule import ServiceIncidentRule
from .service_incident_rule_condition_match_type import ServiceIncidentRuleConditionMatchType
from .service_incident_stakeholder_properties import ServiceIncidentStakeholderProperties
from .service_now_integration import ServiceNowIntegration
from .service_now_v2_callback import ServiceNowV2Callback
from .service_now_v2_integration import ServiceNowV2Integration
from .service_now_v3_callback import ServiceNowV3Callback
from .service_now_v3_integration import ServiceNowV3Integration
from .service_visibility import ServiceVisibility
from .signal_fxv2_integration import SignalFXV2Integration
from .signal_sciences_integration import SignalSciencesIntegration
from .signalfx_integration import SignalfxIntegration
from .site_24x7_integration import Site24X7Integration
from .slack_app_callback import SlackAppCallback
from .slack_app_integration import SlackAppIntegration
from .slack_callback import SlackCallback
from .slack_integration import SlackIntegration
from .snooze_alert_identifier_type import SnoozeAlertIdentifierType
from .snooze_alert_payload import SnoozeAlertPayload
from .soasta_integration import SoastaIntegration
from .solar_winds_web_help_desk_callback import SolarWindsWebHelpDeskCallback
from .solar_winds_web_help_desk_integration import SolarWindsWebHelpDeskIntegration
from .solarwinds_callback import SolarwindsCallback
from .solarwinds_integration import SolarwindsIntegration
from .solarwinds_msp_ncentral_callback import SolarwindsMSPNcentralCallback
from .solarwinds_mspn_central_integration import SolarwindsMSPNCentralIntegration
from .splunk_callback import SplunkCallback
from .splunk_integration import SplunkIntegration
from .splunk_itsi_callback import SplunkITSICallback
from .splunk_itsi_integration import SplunkITSIIntegration
from .stack_storm_integration import StackStormIntegration
from .stackdriver_callback import StackdriverCallback
from .stackdriver_integration import StackdriverIntegration
from .status_cake_integration import StatusCakeIntegration
from .status_hub_integration import StatusHubIntegration
from .status_io_callback import StatusIOCallback
from .status_io_integration import StatusIOIntegration
from .status_page_entry import StatusPageEntry
from .status_page_io_callback import StatusPageIOCallback
from .status_page_io_integration import StatusPageIOIntegration
from .struxure_ware_integration import StruxureWareIntegration
from .success_data import SuccessData
from .success_response import SuccessResponse
from .sumo_logic_callback import SumoLogicCallback
from .sumo_logic_integration import SumoLogicIntegration
from .sysdig_cloud_integration import SysdigCloudIntegration
from .team import Team
from .team_log import TeamLog
from .team_log_details import TeamLogDetails
from .team_member import TeamMember
from .team_meta import TeamMeta
from .team_recipient import TeamRecipient
from .team_responder import TeamResponder
from .team_right import TeamRight
from .team_role import TeamRole
from .team_routing_rule import TeamRoutingRule
from .thousand_eyes_integration import ThousandEyesIntegration
from .threat_stack_integration import ThreatStackIntegration
from .thundra_integration import ThundraIntegration
from .tideways_integration import TidewaysIntegration
from .time_of_day_restriction import TimeOfDayRestriction
from .time_of_day_restriction_interval import TimeOfDayRestrictionInterval
from .time_restriction_interval import TimeRestrictionInterval
from .time_restriction_interval_type import TimeRestrictionIntervalType
from .timeline import Timeline
from .timeline_period import TimelinePeriod
from .timeline_recipient import TimelineRecipient
from .timeline_rotation import TimelineRotation
from .token_based_incoming_feature import TokenBasedIncomingFeature
from .trace_integration import TraceIntegration
from .track_it_callback import TrackItCallback
from .track_it_integration import TrackItIntegration
from .travis_ci_integration import TravisCIIntegration
from .twilio_integration import TwilioIntegration
from .un_acknowledge_alert_identifier_type import UnAcknowledgeAlertIdentifierType
from .update_alert_description_identifier_type import UpdateAlertDescriptionIdentifierType
from .update_alert_description_payload import UpdateAlertDescriptionPayload
from .update_alert_message_identifier_type import UpdateAlertMessageIdentifierType
from .update_alert_message_payload import UpdateAlertMessagePayload
from .update_alert_priority_identifier_type import UpdateAlertPriorityIdentifierType
from .update_alert_priority_payload import UpdateAlertPriorityPayload
from .update_alert_priority_payload_priority import UpdateAlertPriorityPayloadPriority
from .update_contact_payload import UpdateContactPayload
from .update_custom_user_role_identifier_type import UpdateCustomUserRoleIdentifierType
from .update_custom_user_role_payload import UpdateCustomUserRolePayload
from .update_escalation_identifier_type import UpdateEscalationIdentifierType
from .update_escalation_payload import UpdateEscalationPayload
from .update_forwarding_rule_identifier_type import UpdateForwardingRuleIdentifierType
from .update_forwarding_rule_payload import UpdateForwardingRulePayload
from .update_heartbeat_payload import UpdateHeartbeatPayload
from .update_heartbeat_payload_interval_unit import UpdateHeartbeatPayloadIntervalUnit
from .update_heartbeat_response import UpdateHeartbeatResponse
from .update_integration_actions_response import UpdateIntegrationActionsResponse
from .update_integration_response import UpdateIntegrationResponse
from .update_maintenance_payload import UpdateMaintenancePayload
from .update_maintenance_response import UpdateMaintenanceResponse
from .update_notification_rule_payload import UpdateNotificationRulePayload
from .update_notification_rule_response import UpdateNotificationRuleResponse
from .update_notification_rule_step_payload import UpdateNotificationRuleStepPayload
from .update_saved_search_identifier_type import UpdateSavedSearchIdentifierType
from .update_saved_search_payload import UpdateSavedSearchPayload
from .update_schedule_identifier_type import UpdateScheduleIdentifierType
from .update_schedule_override_payload import UpdateScheduleOverridePayload
from .update_schedule_override_response import UpdateScheduleOverrideResponse
from .update_schedule_override_schedule_identifier_type import UpdateScheduleOverrideScheduleIdentifierType
from .update_schedule_payload import UpdateSchedulePayload
from .update_schedule_response import UpdateScheduleResponse
from .update_schedule_rotation_payload import UpdateScheduleRotationPayload
from .update_schedule_rotation_payload_type import UpdateScheduleRotationPayloadType
from .update_schedule_rotation_schedule_identifier_type import UpdateScheduleRotationScheduleIdentifierType
from .update_service_incident_rule_payload import UpdateServiceIncidentRulePayload
from .update_service_incident_rule_payload_condition_match_type import (
    UpdateServiceIncidentRulePayloadConditionMatchType,
)
from .update_service_payload import UpdateServicePayload
from .update_service_payload_visibility import UpdateServicePayloadVisibility
from .update_team_payload import UpdateTeamPayload
from .update_team_role_identifier_type import UpdateTeamRoleIdentifierType
from .update_team_role_payload import UpdateTeamRolePayload
from .update_team_role_team_identifier_type import UpdateTeamRoleTeamIdentifierType
from .update_team_routing_rule_payload import UpdateTeamRoutingRulePayload
from .update_team_routing_rule_team_identifier_type import UpdateTeamRoutingRuleTeamIdentifierType
from .update_user_payload import UpdateUserPayload
from .update_user_payload_details import UpdateUserPayloadDetails
from .updown_io_integration import UpdownIOIntegration
from .uptime_robot_email_integration import UptimeRobotEmailIntegration
from .uptime_robot_integration import UptimeRobotIntegration
from .uptime_webhook_integration import UptimeWebhookIntegration
from .uptrends_email_integration import UptrendsEmailIntegration
from .user import User
from .user_address import UserAddress
from .user_contact import UserContact
from .user_contact_contact_method import UserContactContactMethod
from .user_details import UserDetails
from .user_meta import UserMeta
from .user_recipient import UserRecipient
from .user_responder import UserResponder
from .user_role import UserRole
from .v_center_integration import VCenterIntegration
from .vcsa_integration import VCSAIntegration
from .vivid_cortex_integration import VividCortexIntegration
from .wavefront_integration import WavefrontIntegration
from .weekday_time_restriction import WeekdayTimeRestriction
from .weekday_time_restriction_end_day import WeekdayTimeRestrictionEndDay
from .weekday_time_restriction_interval import WeekdayTimeRestrictionInterval
from .weekday_time_restriction_start_day import WeekdayTimeRestrictionStartDay
from .whats_up_gold_integration import WhatsUpGoldIntegration
from .workato_integration import WorkatoIntegration
from .x_pack_alerting_callback import XPackAlertingCallback
from .x_pack_alerting_integration import XPackAlertingIntegration
from .xl_release_integration import XLReleaseIntegration
from .xmpp_callback import XmppCallback
from .xmpp_integration import XmppIntegration
from .zabbix_callback import ZabbixCallback
from .zabbix_integration import ZabbixIntegration
from .zapier_integration import ZapierIntegration
from .zendesk_callback import ZendeskCallback
from .zendesk_integration import ZendeskIntegration
from .zenoss_callback import ZenossCallback
from .zenoss_integration import ZenossIntegration
from .zyrion_integration import ZyrionIntegration

__all__ = (
    "AccountInfo",
    "AccountPlan",
    "AcknowledgeAlertIdentifierType",
    "ActionCategorized",
    "ActionMapping",
    "ActionMappingAction",
    "ActionMappingMappedAction",
    "AddAttachmentAlertIdentifierType",
    "AddAttachmentBody",
    "AddDetailsIdentifierType",
    "AddDetailsToAlertPayload",
    "AddDetailsToAlertPayloadDetails",
    "AddNoteIdentifierType",
    "AddResponderIdentifierType",
    "AddResponderToAlertPayload",
    "AddTagsIdentifierType",
    "AddTagsToAlertPayload",
    "AddTeamIdentifierType",
    "AddTeamMemberPayload",
    "AddTeamMemberTeamIdentifierType",
    "AddTeamToAlertPayload",
    "AirbrakeIntegration",
    "Alert",
    "AlertActionPayload",
    "AlertAttachment",
    "AlertAttachmentMeta",
    "AlertDetails",
    "AlertFilter",
    "AlertFilterConditionMatchType",
    "AlertIntegration",
    "AlertLog",
    "AlertLogicIntegration",
    "AlertNote",
    "AlertPaging",
    "AlertPolicy",
    "AlertPolicyDetails",
    "AlertPolicyPriority",
    "AlertRecipient",
    "AlertReport",
    "AlertRequestStatus",
    "AlertSiteIntegration",
    "AlertTeamMeta",
    "AlertUserMeta",
    "AmazonCloudTrailIntegration",
    "AmazonEc2AutoScalingIntegration",
    "AmazonRdsIntegration",
    "AmazonRoute53HealthCheckIntegration",
    "AmazonSecurityHubIntegration",
    "AmazonSesIntegration",
    "AmazonSnsCallback",
    "AmazonSnsIntegration",
    "AmazonSnsOutgoingIntegration",
    "ApicaIntegration",
    "ApiIntegration",
    "ApimetricsIntegration",
    "AppDynamicsIntegration",
    "AppOpticsIntegration",
    "AppSignalIntegration",
    "AppSignalV2Integration",
    "AssignAlertIdentifierType",
    "AssignAlertPayload",
    "AtatusIntegration",
    "AtlassianBambooIntegration",
    "AuthenticateIntegrationPayload",
    "AuthenticateIntegrationPayloadType",
    "AutoCloseAction",
    "AutoRestartAction",
    "AutotaskAEMEmailIntegration",
    "AutoTaskEmailIntegration",
    "AzureAutoScaleIntegration",
    "AzureIntegration",
    "AzureOMSIntegration",
    "AzureResourceHealthIntegration",
    "AzureServiceHealthIntegration",
    "BaseAlert",
    "BaseIncident",
    "BaseIncidentExtraProperties",
    "BaseIncomingFeature",
    "BaseIncomingFeatureExtraProperties",
    "BaseIncomingFeatureFeatureType",
    "BaseIntegrationAction",
    "BaseIntegrationActionType",
    "BaseResponse",
    "BaseResponseWithExpandable",
    "BaseWebhookCallbackBaseWebhookCallbackType",
    "BidirectionalCallback",
    "BidirectionalCallbackBidirectionalCallbackType",
    "BidirectionalCallbackNew",
    "BidirectionalCallbackNewBidirectionalCallbackType",
    "BigPandaIntegration",
    "BitbucketIntegration",
    "BlueMatadorIntegration",
    "BMCFootPrintsV11Callback",
    "BMCFootPrintsV11Integration",
    "BMCFootPrintsV12Callback",
    "BMCFootPrintsV12Integration",
    "BMCRemedyCallback",
    "BMCRemedyForceIntegration",
    "BMCRemedyIntegration",
    "BMCRemedyOnDemandCallback",
    "BMCRemedyOnDemandIntegration",
    "BoundaryIntegration",
    "CallbackCondition",
    "CallbackConditionField",
    "CallbackConditionOperation",
    "CampfireCallback",
    "CampfireIntegration",
    "CatchpointIntegration",
    "ChangeNotificationRuleOrderPayload",
    "ChangePolicyOrderPayload",
    "ChangeTeamRoutingRuleOrderPayload",
    "ChangeTeamRoutingRuleOrderTeamIdentifierType",
    "CheckMKIntegration",
    "CherwellCallback",
    "CherwellIntegration",
    "CircleCiIntegration",
    "CirconusCallback",
    "CirconusIntegration",
    "CloseAlertIdentifierType",
    "CloseIncidentIdentifierType",
    "CloudMonixIntegration",
    "CloudSploitIntegration",
    "CloudWatchEventsIntegration",
    "CloudWatchIntegration",
    "CodeshipIntegration",
    "CommonIntegrationAction",
    "ComposeIntegration",
    "Condition",
    "ConditionField",
    "ConditionOperation",
    "ConnectWiseAutomateIntegration",
    "ConnectWiseCallback",
    "ConnectWiseIntegration",
    "ConnectWiseManageCallback",
    "ConnectWiseManageIntegration",
    "ConnectWiseManageV2Callback",
    "ConnectWiseManageV2Integration",
    "ConsulIntegration",
    "Contact",
    "ContactMeta",
    "ContactMetaMethod",
    "ContactStatus",
    "ContactWithApplyOrder",
    "CopperEggIntegration",
    "CountAlertsSearchIdentifierType",
    "CrashlyticsIntegration",
    "CreateAlertPayload",
    "CreateAlertPayloadDetails",
    "CreateAlertPayloadPriority",
    "CreateContactPayload",
    "CreateContactPayloadMethod",
    "CreateCustomUserRolePayload",
    "CreateEscalationPayload",
    "CreateForwardingRulePayload",
    "CreateForwardingRuleResponse",
    "CreateHeartbeatPayload",
    "CreateHeartbeatPayloadAlertPriority",
    "CreateHeartbeatPayloadIntervalUnit",
    "CreateHeartbeatPayloadOwnerTeam",
    "CreateHeartbeatResponse",
    "CreateIncidentPayload",
    "CreateIncidentPayloadDetails",
    "CreateIncidentPayloadPriority",
    "CreateIncidentPayloadStatusPageEntry",
    "CreateIntegrationAction",
    "CreateIntegrationActionExtraProperties",
    "CreateIntegrationActionPriority",
    "CreateIntegrationActionsResponse",
    "CreateIntegrationResponse",
    "CreateMaintenancePayload",
    "CreateMaintenanceResponse",
    "CreateNotificationRulePayload",
    "CreateNotificationRuleResponse",
    "CreateNotificationRuleStepPayload",
    "CreatePolicyResponse",
    "CreateSavedSearchPayload",
    "CreateSavedSearchResponse",
    "CreateScheduleOverridePayload",
    "CreateScheduleOverrideResponse",
    "CreateScheduleOverrideScheduleIdentifierType",
    "CreateSchedulePayload",
    "CreateScheduleResponse",
    "CreateScheduleRotationPayload",
    "CreateScheduleRotationPayloadType",
    "CreateScheduleRotationScheduleIdentifierType",
    "CreateServiceIncidentRulePayload",
    "CreateServiceIncidentRulePayloadConditionMatchType",
    "CreateServicePayload",
    "CreateServicePayloadVisibility",
    "CreateTeamPayload",
    "CreateTeamRolePayload",
    "CreateTeamRoleTeamIdentifierType",
    "CreateTeamRoutingRulePayload",
    "CreateTeamRoutingRuleTeamIdentifierType",
    "CreateUserPayload",
    "CreateUserPayloadDetails",
    "CustomUserRole",
    "DatadogCallback",
    "DatadogIntegration",
    "DataloopIOIntegration",
    "DeduplicationAction",
    "DeduplicationActionDeduplicationActionType",
    "DelayAction",
    "DelayActionDelayOption",
    "DeleteAlertIdentifierType",
    "DeleteCustomUserRoleIdentifierType",
    "DeleteEscalationIdentifierType",
    "DeleteForwardingRuleIdentifierType",
    "DeleteIncidentIdentifierType",
    "DeleteSavedSearchIdentifierType",
    "DeleteScheduleIdentifierType",
    "DeleteScheduleOverrideScheduleIdentifierType",
    "DeleteScheduleRotationScheduleIdentifierType",
    "DeleteTeamIdentifierType",
    "DeleteTeamMemberTeamIdentifierType",
    "DeleteTeamRoleIdentifierType",
    "DeleteTeamRoleTeamIdentifierType",
    "DeleteTeamRoutingRuleTeamIdentifierType",
    "DeprecatedAlertPolicy",
    "DeprecatedAlertPolicyMeta",
    "DeprecatedAlertPolicyMetaType",
    "DeprecatedAlertPolicyType",
    "DeprecatedAutoCloseAlertPolicy",
    "DeprecatedAutoRestartNotificationsAlertPolicy",
    "DeprecatedChangeAlertPolicyOrderPayload",
    "DeprecatedCreateAlertPolicyResponse",
    "DeprecatedGetAlertPolicyResponse",
    "DeprecatedListAlertPoliciesResponse",
    "DeprecatedModifyAlertPolicy",
    "DeprecatedModifyAlertPolicyDetails",
    "DeprecatedModifyAlertPolicyPriority",
    "DeprecatedNotificationDeduplicationAlertPolicy",
    "DeprecatedNotificationDeduplicationAlertPolicyDeduplicationActionType",
    "DeprecatedNotificationDelayAlertPolicy",
    "DeprecatedNotificationDelayAlertPolicyDelayOption",
    "DeskCallback",
    "DeskIntegration",
    "DetectifyIntegration",
    "DisableHeartbeatResponse",
    "DisableIntegrationResponse",
    "DnsCheckIntegration",
    "DripStatIntegration",
    "Duration",
    "DurationTimeUnit",
    "DynatraceAppMonCallback",
    "DynatraceAppMonIntegration",
    "DynatraceIntegration",
    "DynatraceV2Integration",
    "EmailBasedIncomingFeature",
    "EmailIntegration",
    "EnableHeartbeatResponse",
    "EnableIntegrationResponse",
    "ErrorceptionIntegration",
    "ErrorResponse",
    "ErrorResponseResponseHeaders",
    "EscalateAlertIdentifierType",
    "EscalateAlertToNextPayload",
    "Escalation",
    "EscalationRecipient",
    "EscalationRepeat",
    "EscalationRule",
    "EscalationRuleCondition",
    "EscalationRuleNotifyType",
    "EvidentIOIntegration",
    "ExecuteCustomAlertActionIdentifierType",
    "ExportScheduleIdentifierType",
    "Filter",
    "FilterType",
    "FlowdockCallback",
    "FlowdockIntegration",
    "FlowdockV2Callback",
    "FlowdockV2Integration",
    "ForwardingRule",
    "ForwardingRuleMeta",
    "FreshdeskCallback",
    "FreshdeskIntegration",
    "FreshserviceCallback",
    "FreshserviceIntegration",
    "GetAccountInfoResponse",
    "GetAlertAttachmentResponse",
    "GetAlertIdentifierType",
    "GetAlertResponse",
    "GetAttachmentAlertIdentifierType",
    "GetContactResponse",
    "GetCountAlertsResponse",
    "GetCountAlertsResponseData",
    "GetCustomUserRoleIdentifierType",
    "GetCustomUserRoleResponse",
    "GetEscalationIdentifierType",
    "GetEscalationResponse",
    "GetForwardingRuleIdentifierType",
    "GetForwardingRuleResponse",
    "GetHeartbeatResponse",
    "GetIncidentIdentifierType",
    "GetIncidentRequestStatusResponse",
    "GetIncidentResponse",
    "GetIntegrationResponse",
    "GetMaintenanceResponse",
    "GetNextOnCallResponse",
    "GetNextOnCallsScheduleIdentifierType",
    "GetNotificationRuleResponse",
    "GetNotificationRuleStepResponse",
    "GetOnCallResponse",
    "GetOnCallsScheduleIdentifierType",
    "GetPolicyResponse",
    "GetRequestStatusResponse",
    "GetSavedSearchIdentifierType",
    "GetSavedSearchResponse",
    "GetScheduleIdentifierType",
    "GetScheduleOverrideResponse",
    "GetScheduleOverrideScheduleIdentifierType",
    "GetScheduleResponse",
    "GetScheduleRotationResponse",
    "GetScheduleRotationScheduleIdentifierType",
    "GetScheduleTimelineIdentifierType",
    "GetScheduleTimelineIntervalUnit",
    "GetScheduleTimelineResponse",
    "GetServiceAudienceTemplateResponse",
    "GetServiceIncidentRuleResponse",
    "GetServiceResponse",
    "GetTeamIdentifierType",
    "GetTeamResponse",
    "GetTeamRoleIdentifierType",
    "GetTeamRoleResponse",
    "GetTeamRoleTeamIdentifierType",
    "GetTeamRoutingRuleResponse",
    "GetTeamRoutingRuleTeamIdentifierType",
    "GetUserResponse",
    "GhostInspectorIntegration",
    "GitHubIntegration",
    "GitLabIntegration",
    "GoogleStackDriverIntegration",
    "GrafanaIntegration",
    "GrafanaV2Integration",
    "GraylogIntegration",
    "GroupRecipient",
    "Heartbeat",
    "HeartbeatIntegration",
    "HeartbeatIntervalUnit",
    "HeartbeatMeta",
    "HipChatAddOnCallback",
    "HipChatAddOnIntegration",
    "HipChatCallbackV2",
    "HipChatIntegrationV2",
    "HoneybadgerIntegration",
    "HostedGraphiteIntegration",
    "HPServiceManagerCallback",
    "HPServiceManagerIntegration",
    "HumioIntegration",
    "Icinga2Callback",
    "Icinga2Integration",
    "IcingaCallback",
    "IcingaIntegration",
    "Incident",
    "IncidentActionPayload",
    "IncidentRequestStatus",
    "InstanaIntegration",
    "Integration",
    "IntegrationActionFilter",
    "IntegrationActionFilterConditionMatchType",
    "IntegrationCondition",
    "IntegrationConditionOperation",
    "IntegrationMeta",
    "IntegrationType",
    "JenkinsIntegration",
    "JiraCallback",
    "JiraIntegration",
    "JiraServiceDeskCallback",
    "JiraServiceDeskIntegration",
    "KapacitorIntegration",
    "KayakoCallback",
    "KayakoIntegration",
    "LabTechEmailIntegration",
    "LibratoIntegration",
    "LibreNMSCallback",
    "LibreNMSIntegration",
    "LightstepIntegration",
    "ListAlertAttachmentsResponse",
    "ListAlertLogsResponse",
    "ListAlertNotesResponse",
    "ListAlertRecipientsResponse",
    "ListAlertsOrder",
    "ListAlertsResponse",
    "ListAlertsSearchIdentifierType",
    "ListAlertsSort",
    "ListAttachmentsAlertIdentifierType",
    "ListContactsResponse",
    "ListCustomUserRolesResponse",
    "ListEscalationsResponse",
    "ListForwardingRulesResponse",
    "ListHeartbeatResponse",
    "ListHeartbeatResponseData",
    "ListIncidentsOrder",
    "ListIncidentsResponse",
    "ListIncidentsSort",
    "ListIntegrationActionsResponse",
    "ListIntegrationsResponse",
    "ListLogsDirection",
    "ListLogsIdentifierType",
    "ListLogsOrder",
    "ListMaintenanceResponse",
    "ListMaintenanceType",
    "ListNotesDirection",
    "ListNotesIdentifierType",
    "ListNotesOrder",
    "ListNotificationRulesResponse",
    "ListNotificationRuleStepsResponse",
    "ListOnCallsResponse",
    "ListPoliciesResponse",
    "ListRecipientsIdentifierType",
    "ListSavedSearchesResponse",
    "ListScheduleOverrideResponse",
    "ListScheduleOverrideScheduleIdentifierType",
    "ListScheduleRotationsResponse",
    "ListScheduleRotationsScheduleIdentifierType",
    "ListSchedulesResponse",
    "ListServiceIncidentRulesResponse",
    "ListServicesOrder",
    "ListServicesResponse",
    "ListTeamLogsIdentifierType",
    "ListTeamLogsOrder",
    "ListTeamLogsResponse",
    "ListTeamRoleResponse",
    "ListTeamRolesTeamIdentifierType",
    "ListTeamRoutingRulesResponse",
    "ListTeamRoutingRulesTeamIdentifierType",
    "ListTeamsResponse",
    "ListUserEscalationsResponse",
    "ListUserForwardingRulesResponse",
    "ListUserSchedulesResponse",
    "ListUsersOrder",
    "ListUsersResponse",
    "ListUserTeamsResponse",
    "LogentriesIntegration",
    "LogglyIntegration",
    "LogicMonitorCallback",
    "LogicMonitorIntegration",
    "LogstashIntegration",
    "LogzIOIntegration",
    "LookerIntegration",
    "LoomIntegration",
    "MagentrixCallback",
    "MagentrixIntegration",
    "Maintenance",
    "MaintenanceEntity",
    "MaintenanceEntityType",
    "MaintenanceMeta",
    "MaintenanceMetaStatus",
    "MaintenanceRule",
    "MaintenanceRuleState",
    "MaintenanceStatus",
    "MaintenanceTime",
    "MaintenanceTimeType",
    "ManageEngineIntegrationDTO",
    "MaridIntegration",
    "MatchAllConditions",
    "MatchAnyCondition",
    "MattermostCallback",
    "MattermostIntegration",
    "MongoDBCloudIntegration",
    "MonitisEmailIntegration",
    "MonitisIntegration",
    "MSTeamsCallback",
    "MSTeamsIntegration",
    "MSTeamsV2Callback",
    "MSTeamsV2Integration",
    "NagiosBasedV1Callback",
    "NagiosBasedV2Callback",
    "NagiosIntegrationV1",
    "NagiosIntegrationV2",
    "NagiosXIIntegrationV1",
    "NagiosXIIntegrationV2",
    "NagiosXIV1Callback",
    "NagiosXIV2Callback",
    "NetuitiveIntegration",
    "NeustarIntegration",
    "NewRelicIntegration",
    "NewRelicV2Integration",
    "NextOnCall",
    "NodePingIntegration",
    "NotificationActionType",
    "NotificationPolicy",
    "NotificationRepeat",
    "NotificationRule",
    "NotificationRuleMeta",
    "NotificationRuleStep",
    "NotificationRuleStepParent",
    "NotifyTime",
    "ObserviumIntegration",
    "ObserviumV2Integration",
    "OECIntegration",
    "OEMEmailIntegration",
    "OEMIntegration",
    "OnCall",
    "OP5Callback",
    "OP5Integration",
    "OpsDashIntegration",
    "OpsGenieCallback",
    "OpsGenieIntegration",
    "OpsviewIntegration",
    "OutgoingCallback",
    "OutgoingCallbackCallbackType",
    "OutgoingCallbackNew",
    "OutgoingCallbackNewCallbackType",
    "PageDetails",
    "PagerDutyCompatibilityIntegration",
    "PanoptaIntegration",
    "PapertrailIntegration",
    "PartialUpdateServiceAudienceTemplatePayload",
    "Participant",
    "PingdomIntegration",
    "PingdomV2Integration",
    "PingdomWebhookIntegration",
    "PingometerIntegration",
    "PlanioCallback",
    "PlanioIntegration",
    "Policy",
    "PolicyMeta",
    "PolicyMetaType",
    "PolicyType",
    "PrometheusIntegration",
    "PrtgCallback",
    "PrtgIntegration",
    "RackspaceIntegration",
    "RaygunIntegration",
    "Recipient",
    "RecipientType",
    "RedGateSqlMonitorEmailIntegration",
    "RemoveAttachmentAlertIdentifierType",
    "RemoveDetailsIdentifierType",
    "RemoveTagsIdentifierType",
    "Responder",
    "ResponderType",
    "RiemannIntegration",
    "RigorIntegration",
    "RingCentralEmailIntegration",
    "RollbarCallback",
    "RollbarIntegration",
    "RunscopeIntegration",
    "SalesForceServiceCloudCallback",
    "SalesForceServiceCloudIntegration",
    "SaltStackIntegration",
    "SavedSearch",
    "SavedSearchEntity",
    "SavedSearchMeta",
    "ScalyrIntegration",
    "Schedule",
    "ScheduleMeta",
    "ScheduleOverride",
    "ScheduleOverrideMeta",
    "ScheduleOverrideRotation",
    "ScheduleRecipient",
    "ScheduleRotation",
    "ScheduleRotationType",
    "ScheduleRotationWithParent",
    "ScheduleTimeline",
    "ScomIntegration",
    "ScoutIntegration",
    "SematextSpmIntegration",
    "SensuIntegration",
    "SentryIntegration",
    "ServerDensityIntegration",
    "ServerGuard24Integration",
    "Service",
    "ServiceAudienceTemplate",
    "ServiceAudienceTemplateCondition",
    "ServiceAudienceTemplateConditionMatchField",
    "ServiceAudienceTemplateResponder",
    "ServiceAudienceTemplateStakeholder",
    "ServiceAudienceTemplateStakeholderConditionMatchType",
    "ServiceIncidentCondition",
    "ServiceIncidentConditionField",
    "ServiceIncidentConditionOperation",
    "ServiceIncidentProperties",
    "ServiceIncidentPropertiesDetails",
    "ServiceIncidentPropertiesPriority",
    "ServiceIncidentRule",
    "ServiceIncidentRuleConditionMatchType",
    "ServiceIncidentStakeholderProperties",
    "ServiceNowIntegration",
    "ServiceNowV2Callback",
    "ServiceNowV2Integration",
    "ServiceNowV3Callback",
    "ServiceNowV3Integration",
    "ServiceVisibility",
    "SignalfxIntegration",
    "SignalFXV2Integration",
    "SignalSciencesIntegration",
    "Site24X7Integration",
    "SlackAppCallback",
    "SlackAppIntegration",
    "SlackCallback",
    "SlackIntegration",
    "SnoozeAlertIdentifierType",
    "SnoozeAlertPayload",
    "SoastaIntegration",
    "SolarwindsCallback",
    "SolarwindsIntegration",
    "SolarwindsMSPNcentralCallback",
    "SolarwindsMSPNCentralIntegration",
    "SolarWindsWebHelpDeskCallback",
    "SolarWindsWebHelpDeskIntegration",
    "SplunkCallback",
    "SplunkIntegration",
    "SplunkITSICallback",
    "SplunkITSIIntegration",
    "StackdriverCallback",
    "StackdriverIntegration",
    "StackStormIntegration",
    "StatusCakeIntegration",
    "StatusHubIntegration",
    "StatusIOCallback",
    "StatusIOIntegration",
    "StatusPageEntry",
    "StatusPageIOCallback",
    "StatusPageIOIntegration",
    "StruxureWareIntegration",
    "SuccessData",
    "SuccessResponse",
    "SumoLogicCallback",
    "SumoLogicIntegration",
    "SysdigCloudIntegration",
    "Team",
    "TeamLog",
    "TeamLogDetails",
    "TeamMember",
    "TeamMeta",
    "TeamRecipient",
    "TeamResponder",
    "TeamRight",
    "TeamRole",
    "TeamRoutingRule",
    "ThousandEyesIntegration",
    "ThreatStackIntegration",
    "ThundraIntegration",
    "TidewaysIntegration",
    "Timeline",
    "TimelinePeriod",
    "TimelineRecipient",
    "TimelineRotation",
    "TimeOfDayRestriction",
    "TimeOfDayRestrictionInterval",
    "TimeRestrictionInterval",
    "TimeRestrictionIntervalType",
    "TokenBasedIncomingFeature",
    "TraceIntegration",
    "TrackItCallback",
    "TrackItIntegration",
    "TravisCIIntegration",
    "TwilioIntegration",
    "UnAcknowledgeAlertIdentifierType",
    "UpdateAlertDescriptionIdentifierType",
    "UpdateAlertDescriptionPayload",
    "UpdateAlertMessageIdentifierType",
    "UpdateAlertMessagePayload",
    "UpdateAlertPriorityIdentifierType",
    "UpdateAlertPriorityPayload",
    "UpdateAlertPriorityPayloadPriority",
    "UpdateContactPayload",
    "UpdateCustomUserRoleIdentifierType",
    "UpdateCustomUserRolePayload",
    "UpdateEscalationIdentifierType",
    "UpdateEscalationPayload",
    "UpdateForwardingRuleIdentifierType",
    "UpdateForwardingRulePayload",
    "UpdateHeartbeatPayload",
    "UpdateHeartbeatPayloadIntervalUnit",
    "UpdateHeartbeatResponse",
    "UpdateIntegrationActionsResponse",
    "UpdateIntegrationResponse",
    "UpdateMaintenancePayload",
    "UpdateMaintenanceResponse",
    "UpdateNotificationRulePayload",
    "UpdateNotificationRuleResponse",
    "UpdateNotificationRuleStepPayload",
    "UpdateSavedSearchIdentifierType",
    "UpdateSavedSearchPayload",
    "UpdateScheduleIdentifierType",
    "UpdateScheduleOverridePayload",
    "UpdateScheduleOverrideResponse",
    "UpdateScheduleOverrideScheduleIdentifierType",
    "UpdateSchedulePayload",
    "UpdateScheduleResponse",
    "UpdateScheduleRotationPayload",
    "UpdateScheduleRotationPayloadType",
    "UpdateScheduleRotationScheduleIdentifierType",
    "UpdateServiceIncidentRulePayload",
    "UpdateServiceIncidentRulePayloadConditionMatchType",
    "UpdateServicePayload",
    "UpdateServicePayloadVisibility",
    "UpdateTeamPayload",
    "UpdateTeamRoleIdentifierType",
    "UpdateTeamRolePayload",
    "UpdateTeamRoleTeamIdentifierType",
    "UpdateTeamRoutingRulePayload",
    "UpdateTeamRoutingRuleTeamIdentifierType",
    "UpdateUserPayload",
    "UpdateUserPayloadDetails",
    "UpdownIOIntegration",
    "UptimeRobotEmailIntegration",
    "UptimeRobotIntegration",
    "UptimeWebhookIntegration",
    "UptrendsEmailIntegration",
    "User",
    "UserAddress",
    "UserContact",
    "UserContactContactMethod",
    "UserDetails",
    "UserMeta",
    "UserRecipient",
    "UserResponder",
    "UserRole",
    "VCenterIntegration",
    "VCSAIntegration",
    "VividCortexIntegration",
    "WavefrontIntegration",
    "WeekdayTimeRestriction",
    "WeekdayTimeRestrictionEndDay",
    "WeekdayTimeRestrictionInterval",
    "WeekdayTimeRestrictionStartDay",
    "WhatsUpGoldIntegration",
    "WorkatoIntegration",
    "XLReleaseIntegration",
    "XmppCallback",
    "XmppIntegration",
    "XPackAlertingCallback",
    "XPackAlertingIntegration",
    "ZabbixCallback",
    "ZabbixIntegration",
    "ZapierIntegration",
    "ZendeskCallback",
    "ZendeskIntegration",
    "ZenossCallback",
    "ZenossIntegration",
    "ZyrionIntegration",
)



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

if TYPE_CHECKING:
    from ..models.user_meta import UserMeta


T = TypeVar("T", bound="UpdateForwardingRulePayload")


@_attrs_define
class UpdateForwardingRulePayload:
    """
    Attributes:
        from_user (UserMeta):
        to_user (UserMeta):
        start_date (datetime.datetime): The date and time for forwarding will start
        end_date (datetime.datetime): The date and time for forwarding will end
    """

    from_user: "UserMeta"
    to_user: "UserMeta"
    start_date: datetime.datetime
    end_date: datetime.datetime
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from_user = self.from_user.to_dict()

        to_user = self.to_user.to_dict()

        start_date = self.start_date.isoformat()

        end_date = self.end_date.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "fromUser": from_user,
                "toUser": to_user,
                "startDate": start_date,
                "endDate": end_date,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.user_meta import UserMeta

        d = src_dict.copy()
        from_user = UserMeta.from_dict(d.pop("fromUser"))

        to_user = UserMeta.from_dict(d.pop("toUser"))

        start_date = isoparse(d.pop("startDate"))

        end_date = isoparse(d.pop("endDate"))

        update_forwarding_rule_payload = cls(
            from_user=from_user,
            to_user=to_user,
            start_date=start_date,
            end_date=end_date,
        )

        update_forwarding_rule_payload.additional_properties = d
        return update_forwarding_rule_payload

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
